@@ -51,7 +51,8 @@ void error_message(gchar *message) {
   /* Ensure that the dialog box is destroyed when the user clicks ok. */
      
   gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy), dialog);
+			     GTK_SIGNAL_FUNC (gtk_widget_destroy), 
+			     GTK_OBJECT(dialog));
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
 		     okay_button);
 
@@ -293,7 +294,7 @@ static void cb_select_child (GtkWidget *root_tree, GtkWidget *child,
 
       gtk_clist_append(GTK_CLIST(clist), rowdata);
 
-      (char *)dirp += dirlen;
+      dirp = (struct smbc_dirent *) ((char *) dirp + dirlen);
       err -= dirlen;
 
     }
@@ -428,7 +429,7 @@ static void cb_itemsignal( GtkWidget *item,
 
 	}
 
-	(char *)dirp += dirlen;
+        dirp = (struct smbc_dirent *) ((char *) dirp + dirlen);
 	err -= dirlen;
 
       }
@@ -563,7 +564,7 @@ static void cb_wholenet(GtkWidget *item, gchar *signame)
 	gtk_signal_connect(GTK_OBJECT(subtree), "unselect_child",
 			   GTK_SIGNAL_FUNC(cb_unselect_child), real_tree);
 
-	(char *)dirp += dirlen;
+        dirp = (struct smbc_dirent *) ((char *) dirp + dirlen);
 	err -= dirlen;
 
       }
@@ -705,9 +706,9 @@ int main( int   argc,
 
   /* Now, build the top level display ... */
 
-  if ((dh = smbc_opendir("smb:///")) < 0) {
+  if ((dh = smbc_opendir("smb://")) < 0) {
 
-    fprintf(stderr, "Could not list default workgroup: smb:///: %s\n",
+    fprintf(stderr, "Could not list workgroups: smb://: %s\n",
 	    strerror(errno));
 
     exit(1);
@@ -744,8 +745,6 @@ int main( int   argc,
 
   /* Now, get the items in smb:/// and add them to the tree */
 
-  dirp = (struct smbc_dirent *)dirbuf;
-
   while ((err = smbc_getdents(dh, (struct smbc_dirent *)dirbuf, 
 			      sizeof(dirbuf))) != 0) {
 
@@ -758,6 +757,8 @@ int main( int   argc,
 
     }
 
+    dirp = (struct smbc_dirent *)dirbuf;
+
     fprintf(stdout, "Dir len: %u\n", err);
 
     while (err > 0) { /* Extract each entry and make a sub-tree */
@@ -796,7 +797,7 @@ int main( int   argc,
       gtk_signal_connect (GTK_OBJECT(subtree), "unselect_child",
 			  GTK_SIGNAL_FUNC(cb_unselect_child), tree);
 
-      (char *)dirp += dirlen;
+      dirp = (struct smbc_dirent *) ((char *) dirp + dirlen);
       err -= dirlen;
 
     }
