@@ -20,9 +20,9 @@
 */
 
 /* this defines the charset types used in samba */
-typedef enum {CH_UCS2=0, CH_UNIX=1, CH_DISPLAY=2, CH_DOS=3, CH_UTF8=4} charset_t;
+typedef enum {CH_UTF16LE=0, CH_UTF16=0, CH_UNIX=1, CH_DISPLAY=2, CH_DOS=3, CH_UTF8=4, CH_UTF16BE=5} charset_t;
 
-#define NUM_CHARSETS 5
+#define NUM_CHARSETS 6
 
 /* 
  *   for each charset we have a function that pushes from that charset to a ucs2
@@ -31,9 +31,9 @@ typedef enum {CH_UCS2=0, CH_UNIX=1, CH_DISPLAY=2, CH_DOS=3, CH_UTF8=4} charset_t
 
 struct charset_functions {
 	const char *name;
-	size_t (*pull)(void *, char **inbuf, size_t *inbytesleft,
+	size_t (*pull)(void *, const char **inbuf, size_t *inbytesleft,
 				   char **outbuf, size_t *outbytesleft);
-	size_t (*push)(void *, char **inbuf, size_t *inbytesleft,
+	size_t (*push)(void *, const char **inbuf, size_t *inbytesleft,
 				   char **outbuf, size_t *outbytesleft);
 	struct charset_functions *prev, *next;
 };
@@ -57,7 +57,7 @@ struct charset_gap_table {
  *
  *     */
 #define SMB_GENERATE_CHARSET_MODULE_8_BIT_GAP(CHARSETNAME) 					\
-static size_t CHARSETNAME ## _push(void *cd, char **inbuf, size_t *inbytesleft,			\
+static size_t CHARSETNAME ## _push(void *cd, const char **inbuf, size_t *inbytesleft,			\
 			 char **outbuf, size_t *outbytesleft) 					\
 { 												\
 	while (*inbytesleft >= 2 && *outbytesleft >= 1) { 					\
@@ -97,7 +97,7 @@ static size_t CHARSETNAME ## _push(void *cd, char **inbuf, size_t *inbytesleft,
 	return 0;										\
 }												\
 												\
-static size_t CHARSETNAME ## _pull(void *cd, char **inbuf, size_t *inbytesleft,				\
+static size_t CHARSETNAME ## _pull(void *cd, const char **inbuf, size_t *inbytesleft,				\
 			 char **outbuf, size_t *outbytesleft)					\
 {												\
 	while (*inbytesleft >= 1 && *outbytesleft >= 2) {					\
@@ -119,9 +119,9 @@ static size_t CHARSETNAME ## _pull(void *cd, char **inbuf, size_t *inbytesleft,
 struct charset_functions CHARSETNAME ## _functions = 						\
 		{#CHARSETNAME, CHARSETNAME ## _pull, CHARSETNAME ## _push};			\
 												\
+NTSTATUS charset_ ## CHARSETNAME ## _init(void);							\
 NTSTATUS charset_ ## CHARSETNAME ## _init(void)							\
 {												\
 	return smb_register_charset(& CHARSETNAME ## _functions);				\
 }												\
 
-
