@@ -1,7 +1,8 @@
 /* 
-   Unix SMB/CIFS implementation.
+   Samba's Internal Registry objects
+   
    SMB parameters and setup
-   Copyright (C) Gerald Carter                   2002-2005.
+   Copyright (C) Gerald Carter                   2002-2006.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +35,6 @@ typedef struct {
 /* container for registry values */
 
 typedef struct {
-	TALLOC_CTX      *ctx;
 	uint32          num_values;
 	REGISTRY_VALUE	**values;
 } REGVAL_CTR;
@@ -42,7 +42,6 @@ typedef struct {
 /* container for registry subkey names */
 
 typedef struct {
-	TALLOC_CTX	*ctx;
 	uint32          num_subkeys;
 	char            **subkeys;
 } REGSUBKEY_CTR;
