@@ -1,10 +1,10 @@
 /* 
    Unix SMB/Netbios implementation.
-   Version 1.9.
-   SMB parameters and setup
+
    Copyright (C) Andrew Tridgell              1992-2000,
    Copyright (C) Luke Kenneth Casson Leighton 1996-2000,
    Copyright (C) Jean Francois Micouleau      1998-2000.
+   Copyright (C) Gerald Carter                2001-2006.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,9 +24,6 @@
 #ifndef _RPC_SPOOLSS_H		/* _RPC_SPOOLSS_H */
 #define _RPC_SPOOLSS_H
 
-#define INTEGER 1
-#define STRING 2
-
 /* spoolss pipe: this are the calls which are not implemented ...
 #define SPOOLSS_GETPRINTERDRIVER			0x0b
 #define SPOOLSS_READPRINTER				0x16
@@ -52,9 +49,6 @@
 #define SPOOLSS_REMOTEFINDFIRSTPRINTERCHANGENOTIFICATION0x3e
 #define SPOOLSS_SPOOLERINIT				0x3f
 #define SPOOLSS_RESETPRINTEREX				0x40
-#define SPOOLSS_DELETEPRINTERDATAEX			0x51
-#define SPOOLSS_DELETEPRINTERDRIVEREX			0x54
-#define SPOOLSS_ADDPRINTERDRIVEREX			0x59
 */
 
 /* those are implemented */
@@ -110,6 +104,21 @@
 #define SPOOLSS_GETPRINTERDATAEX			0x4e
 #define SPOOLSS_ENUMPRINTERDATAEX			0x4f
 #define SPOOLSS_ENUMPRINTERKEY				0x50
+#define SPOOLSS_DELETEPRINTERDATAEX			0x51
+#define SPOOLSS_DELETEPRINTERKEY			0x52
+#define SPOOLSS_DELETEPRINTERDRIVEREX			0x54
+#define SPOOLSS_XCVDATAPORT				0x58
+#define SPOOLSS_ADDPRINTERDRIVEREX			0x59
+
+/* 
+ * Special strings for the OpenPrinter() call.  See the MSDN DDK
+ * docs on the XcvDataPort() for more details.
+ */
+
+#define SPL_LOCAL_PORT            "Local Port"
+#define SPL_TCPIP_PORT            "Standard TCP/IP Port"
+#define SPL_XCV_MONITOR_LOCALMON  ",XcvMonitor Local Port"
+#define SPL_XCV_MONITOR_TCPMON    ",XcvMonitor Standard TCP/IP Port"
 
 
 #define PRINTER_CONTROL_UNPAUSE		0x00000000
@@ -118,6 +127,7 @@
 #define PRINTER_CONTROL_PURGE		0x00000003
 #define PRINTER_CONTROL_SET_STATUS	0x00000004
 
+#define PRINTER_STATUS_OK               0x00000000
 #define PRINTER_STATUS_PAUSED		0x00000001
 #define PRINTER_STATUS_ERROR		0x00000002
 #define PRINTER_STATUS_PENDING_DELETION	0x00000004
@@ -155,27 +165,22 @@
 #define PRINTER_ACCESS_ADMINISTER	0x00000004
 #define PRINTER_ACCESS_USE		0x00000008
 #define JOB_ACCESS_ADMINISTER		0x00000010
+#define JOB_ACCESS_READ			0x00000020
 
 /* JOB status codes. */
 
-#define JOB_STATUS_PAUSED		0x001
-#define JOB_STATUS_ERROR		0x002
-#define JOB_STATUS_DELETING		0x004
-#define JOB_STATUS_SPOOLING		0x008
-#define JOB_STATUS_PRINTING		0x010
-#define JOB_STATUS_OFFLINE		0x020
-#define JOB_STATUS_PAPEROUT		0x040
-#define JOB_STATUS_PRINTED		0x080
-#define JOB_STATUS_DELETED		0x100
-#define JOB_STATUS_BLOCKED		0x200
-#define JOB_STATUS_USER_INTERVENTION	0x400
-
-/* ACE masks for the various print permissions */
-
-#define PRINTER_ACE_FULL_CONTROL      GENERIC_ALL_ACCESS
-#define PRINTER_ACE_MANAGE_DOCUMENTS  READ_CONTROL_ACCESS
-#define PRINTER_ACE_PRINT             \
-    (GENERIC_READ_ACCESS | GENERIC_WRITE_ACCESS | GENERIC_EXECUTE_ACCESS)
+#define JOB_STATUS_QUEUED               0x0000
+#define JOB_STATUS_PAUSED		0x0001
+#define JOB_STATUS_ERROR		0x0002
+#define JOB_STATUS_DELETING		0x0004
+#define JOB_STATUS_SPOOLING		0x0008
+#define JOB_STATUS_PRINTING		0x0010
+#define JOB_STATUS_OFFLINE		0x0020
+#define JOB_STATUS_PAPEROUT		0x0040
+#define JOB_STATUS_PRINTED		0x0080
+#define JOB_STATUS_DELETED		0x0100
+#define JOB_STATUS_BLOCKED		0x0200
+#define JOB_STATUS_USER_INTERVENTION	0x0400
 
 /* Access rights for print servers */
 #define SERVER_ALL_ACCESS	STANDARD_RIGHTS_REQUIRED_ACCESS|SERVER_ACCESS_ADMINISTER|SERVER_ACCESS_ENUMERATE
@@ -190,17 +195,29 @@
 #define PRINTER_EXECUTE       STANDARD_RIGHTS_EXECUTE_ACCESS|PRINTER_ACCESS_USE
 
 /* Access rights for jobs */
-#define JOB_ALL_ACCESS	STANDARD_RIGHTS_REQUIRED_ACCESS|JOB_ACCESS_ADMINISTER
-#define JOB_READ	STANDARD_RIGHTS_READ_ACCESS|JOB_ACCESS_ADMINISTER
-#define JOB_WRITE	STANDARD_RIGHTS_WRITE_ACCESS|JOB_ACCESS_ADMINISTER
-#define JOB_EXECUTE	STANDARD_RIGHTS_EXECUTE_ACCESS|JOB_ACCESS_ADMINISTER
+#define JOB_ALL_ACCESS	STANDARD_RIGHTS_REQUIRED_ACCESS|JOB_ACCESS_ADMINISTER|JOB_ACCESS_READ|PRINTER_ACCESS_USE
+#define JOB_READ	STANDARD_RIGHTS_READ_ACCESS|JOB_ACCESS_ADMINISTER|JOB_ACCESS_READ
+#define JOB_WRITE	STANDARD_RIGHTS_WRITE_ACCESS|JOB_ACCESS_ADMINISTER|PRINTER_ACCESS_USE
+#define JOB_EXECUTE	STANDARD_RIGHTS_EXECUTE_ACCESS|JOB_ACCESS_ADMINISTER|PRINTER_ACCESS_USE
 
-#define ONE_VALUE 1
-#define TWO_VALUE 2
-#define POINTER   3
+/* ACE masks for the various print permissions */
+
+#define PRINTER_ACE_FULL_CONTROL      (GENERIC_ALL_ACCESS|PRINTER_ALL_ACCESS)
+#define PRINTER_ACE_MANAGE_DOCUMENTS  (GENERIC_ALL_ACCESS|READ_CONTROL_ACCESS)
+#define PRINTER_ACE_PRINT             (GENERIC_EXECUTE_ACCESS|READ_CONTROL_ACCESS|PRINTER_ACCESS_USE)
+
+
+/* Notify field types */
+
+#define NOTIFY_ONE_VALUE 1		/* Notify data is stored in value1 */
+#define NOTIFY_TWO_VALUE 2		/* Notify data is stored in value2 */
+#define NOTIFY_POINTER   3		/* Data is a pointer to a buffer */
+#define NOTIFY_STRING    4		/* Data is a pointer to a buffer w/length */
+#define NOTIFY_SECDESC   5		/* Data is a security descriptor */
 
 #define PRINTER_NOTIFY_TYPE 0x00
 #define JOB_NOTIFY_TYPE     0x01
+#define PRINT_TABLE_END     0xFF
 
 #define MAX_PRINTER_NOTIFY 26
 #define MAX_JOB_NOTIFY 24
@@ -366,6 +383,12 @@ PRINTER_MESSAGE_INFO;
 #define PRINTER_ATTRIBUTE_ENABLE_BIDI		0x00000800
 
 #define PRINTER_ATTRIBUTE_RAW_ONLY		0x00001000
+#define PRINTER_ATTRIBUTE_PUBLISHED		0x00002000
+
+#define PRINTER_ATTRIBUTE_SAMBA			(PRINTER_ATTRIBUTE_RAW_ONLY|\
+						 PRINTER_ATTRIBUTE_SHARED|\
+						 PRINTER_ATTRIBUTE_LOCAL)
+#define PRINTER_ATTRIBUTE_NOT_SAMBA		(PRINTER_ATTRIBUTE_NETWORK)
 
 #define NO_PRIORITY	 0
 #define MAX_PRIORITY	99
@@ -396,29 +419,41 @@ PRINTER_MESSAGE_INFO;
 #define PRINTER_ENUM_ICON7		0x00400000
 #define PRINTER_ENUM_ICON8		0x00800000
 
+/* FLAGS for SPOOLSS_DELETEPRINTERDRIVEREX */
+
+#define DPD_DELETE_UNUSED_FILES		0x00000001
+#define DPD_DELETE_SPECIFIC_VERSION	0x00000002
+#define DPD_DELETE_ALL_FILES		0x00000004
+
+#define DRIVER_ANY_VERSION		0xffffffff
+#define DRIVER_MAX_VERSION		4
+
+/* FLAGS for SPOOLSS_ADDPRINTERDRIVEREX */
+
+#define APD_STRICT_UPGRADE		0x00000001
+#define APD_STRICT_DOWNGRADE		0x00000002
+#define APD_COPY_ALL_FILES		0x00000004
+#define APD_COPY_NEW_FILES		0x00000008
+
+
 /* this struct is undocumented */
 /* thanks to the ddk ... */
-typedef struct spool_user_1
-{
+typedef struct {
 	uint32 size;		/* length of user_name & client_name + 2? */
-	uint32 client_name_ptr;
-	uint32 user_name_ptr;
+	UNISTR2 *client_name;
+	UNISTR2 *user_name;
 	uint32 build;
 	uint32 major;
 	uint32 minor;
 	uint32 processor;
-	UNISTR2 client_name;
-	UNISTR2 user_name;
-}
-SPOOL_USER_1;
+} SPOOL_USER_1;
 
-typedef struct spool_user_ctr_info
-{
+typedef struct {
 	uint32 level;
-	uint32 ptr;
-	SPOOL_USER_1 user1;
-}
-SPOOL_USER_CTR;
+	union {
+		SPOOL_USER_1 *user1;
+	} user;
+} SPOOL_USER_CTR;
 
 /*
  * various bits in the DEVICEMODE.fields member
@@ -494,7 +529,7 @@ typedef struct devicemode
 	uint32 reserved2;
 	uint32 panningwidth;
 	uint32 panningheight;
-	uint8 *private;
+	uint8 *dev_private;
 }
 DEVICEMODE;
 
@@ -515,41 +550,33 @@ typedef struct _printer_default
 }
 PRINTER_DEFAULT;
 
-/* SPOOL_Q_OPEN_PRINTER request to open a printer */
-typedef struct spool_q_open_printer
-{
-	uint32 printername_ptr;
-	UNISTR2 printername;
-	PRINTER_DEFAULT printer_default;
-}
-SPOOL_Q_OPEN_PRINTER;
+/********************************************/
 
-/* SPOOL_R_OPEN_PRINTER reply to an open printer */
-typedef struct spool_r_open_printer
-{
+typedef struct {
+	UNISTR2 *printername;
+	PRINTER_DEFAULT printer_default;
+} SPOOL_Q_OPEN_PRINTER;
+
+typedef struct {
 	POLICY_HND handle;	/* handle used along all transactions (20*uint8) */
 	WERROR status;
-}
-SPOOL_R_OPEN_PRINTER;
+} SPOOL_R_OPEN_PRINTER;
 
-/* SPOOL_Q_OPEN_PRINTER_EX request to open a printer */
-typedef struct spool_q_open_printer_ex
-{
-	uint32 printername_ptr;
-	UNISTR2 printername;
+/********************************************/
+
+typedef struct {
+	UNISTR2 *printername;
 	PRINTER_DEFAULT printer_default;
 	uint32 user_switch;
 	SPOOL_USER_CTR user_ctr;
-}
-SPOOL_Q_OPEN_PRINTER_EX;
+} SPOOL_Q_OPEN_PRINTER_EX;
 
-/* SPOOL_R_OPEN_PRINTER_EX reply to an open printer */
-typedef struct spool_r_open_printer_ex
-{
+typedef struct {
 	POLICY_HND handle;	/* handle used along all transactions (20*uint8) */
 	WERROR status;
-}
-SPOOL_R_OPEN_PRINTER_EX;
+} SPOOL_R_OPEN_PRINTER_EX;
+
+/********************************************/
 
 typedef struct spool_notify_option_type
 {
@@ -585,15 +612,6 @@ typedef struct s_header_type
 }
 HEADER_TYPE;
 
-typedef struct new_buffer
-{
-	uint32 ptr;
-	uint32 size;
-	prs_struct prs;
-	uint32 struct_start;
-	uint32 string_at_end;
-}
-NEW_BUFFER;
 
 typedef struct spool_q_getprinterdata
 {
@@ -679,6 +697,23 @@ typedef struct spool_r_deleteprinterdriver
 }
 SPOOL_R_DELETEPRINTERDRIVER;
 
+typedef struct spool_q_deleteprinterdriverex
+{
+	uint32 server_ptr;
+	UNISTR2 server;
+	UNISTR2 arch;
+	UNISTR2 driver;
+	uint32 delete_flags;
+	uint32 version;
+}
+SPOOL_Q_DELETEPRINTERDRIVEREX;
+
+typedef struct spool_r_deleteprinterdriverex
+{
+	WERROR status;
+}
+SPOOL_R_DELETEPRINTERDRIVEREX;
+
 
 typedef struct spool_doc_info_1
 {
@@ -763,19 +798,20 @@ typedef struct spool_notify_info_data
 	uint16 field;
 	uint32 reserved;
 	uint32 id;
-	union
-	{
+	union {
 		uint32 value[2];
-		struct
-		{
+		struct {
 			uint32 length;
 			uint16 *string;
-		}
-		data;
+		} data;
+		struct {
+			uint32	size;
+			SEC_DESC *desc;
+		} sd;
 	}
 	notify_data;
 	uint32 size;
-	BOOL enc_type;
+	uint32 enc_type;
 } SPOOL_NOTIFY_INFO_DATA;
 
 typedef struct spool_notify_info
@@ -926,7 +962,6 @@ PRINTER_INFO_2;
 
 typedef struct printer_info_3
 {
-	uint32 flags;
 	SEC_DESC *secdesc;
 }
 PRINTER_INFO_3;
@@ -949,13 +984,31 @@ typedef struct printer_info_5
 }
 PRINTER_INFO_5;
 
+typedef struct printer_info_6
+{
+	uint32 status;
+}
+PRINTER_INFO_6;
+
+#define SPOOL_DS_PUBLISH	1
+#define SPOOL_DS_UPDATE		2
+#define SPOOL_DS_UNPUBLISH	4
+#define SPOOL_DS_PENDING        0x80000000
+
+typedef struct printer_info_7
+{
+	UNISTR guid; /* text form of printer guid */
+	uint32 action;
+}
+PRINTER_INFO_7;
+
 typedef struct spool_q_enumprinters
 {
 	uint32 flags;
 	uint32 servername_ptr;
 	UNISTR2 servername;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMPRINTERS;
@@ -968,12 +1021,13 @@ typedef struct printer_info_ctr_info
 	PRINTER_INFO_3 *printers_3;
 	PRINTER_INFO_4 *printers_4;
 	PRINTER_INFO_5 *printers_5;
+	PRINTER_INFO_7 *printers_7;
 }
 PRINTER_INFO_CTR;
 
 typedef struct spool_r_enumprinters
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;		/* bytes needed */
 	uint32 returned;	/* number of printers */
 	WERROR status;
@@ -985,7 +1039,7 @@ typedef struct spool_q_getprinter
 {
 	POLICY_HND handle;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_GETPRINTER;
@@ -1003,7 +1057,7 @@ typedef struct printer_info_info
 
 typedef struct spool_r_getprinter
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	WERROR status;
 } SPOOL_R_GETPRINTER;
@@ -1077,7 +1131,7 @@ typedef struct spool_q_getprinterdriver2
 	uint32 architecture_ptr;
 	UNISTR2 architecture;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 	uint32 clientmajorversion;
 	uint32 clientminorversion;
@@ -1086,7 +1140,7 @@ SPOOL_Q_GETPRINTERDRIVER2;
 
 typedef struct spool_r_getprinterdriver2
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 servermajorversion;
 	uint32 serverminorversion;
@@ -1107,14 +1161,14 @@ typedef struct spool_q_addjob
 {
 	POLICY_HND handle;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ADDJOB;
 
 typedef struct spool_r_addjob
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	WERROR status;
 }
@@ -1191,7 +1245,7 @@ typedef struct spool_q_enumjobs
 	uint32 firstjob;
 	uint32 numofjobs;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMJOBS;
@@ -1200,8 +1254,8 @@ typedef struct job_info_ctr_info
 {
 	union
 	{
-		JOB_INFO_1 **job_info_1;
-		JOB_INFO_2 **job_info_2;
+		JOB_INFO_1 *job_info_1;
+		JOB_INFO_2 *job_info_2;
 		void *info;
 	} job;
 
@@ -1209,7 +1263,7 @@ typedef struct job_info_ctr_info
 
 typedef struct spool_r_enumjobs
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 returned;
 	WERROR status;
@@ -1245,12 +1299,18 @@ typedef struct s_port_info_2
 }
 PORT_INFO_2;
 
+/* Port Type bits */
+#define PORT_TYPE_WRITE         0x0001
+#define PORT_TYPE_READ          0x0002
+#define PORT_TYPE_REDIRECTED    0x0004
+#define PORT_TYPE_NET_ATTACHED  0x0008
+
 typedef struct spool_q_enumports
 {
 	uint32 name_ptr;
 	UNISTR2 name;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMPORTS;
@@ -1269,7 +1329,7 @@ PORT_INFO_CTR;
 
 typedef struct spool_r_enumports
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;		/* bytes needed */
 	uint32 returned;	/* number of printers */
 	WERROR status;
@@ -1319,14 +1379,14 @@ typedef struct spool_q_enumprinterdrivers
 	uint32 environment_ptr;
 	UNISTR2 environment;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMPRINTERDRIVERS;
 
 typedef struct spool_r_enumprinterdrivers
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 returned;
 	WERROR status;
@@ -1354,14 +1414,14 @@ typedef struct spool_q_enumforms
 {
 	POLICY_HND handle;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMFORMS;
 
 typedef struct spool_r_enumforms
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 numofforms;
 	WERROR status;
@@ -1373,14 +1433,14 @@ typedef struct spool_q_getform
 	POLICY_HND handle;
 	UNISTR2 formname;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_GETFORM;
 
 typedef struct spool_r_getform
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	WERROR status;
 }
@@ -1440,6 +1500,14 @@ typedef struct spool_printer_info_level_3
 }
 SPOOL_PRINTER_INFO_LEVEL_3;
 
+typedef struct spool_printer_info_level_7
+{
+	uint32 guid_ptr;
+	uint32 action;
+	UNISTR2 guid;
+}
+SPOOL_PRINTER_INFO_LEVEL_7;
+
 typedef struct spool_printer_info_level
 {
 	uint32 level;
@@ -1447,6 +1515,7 @@ typedef struct spool_printer_info_level
 	SPOOL_PRINTER_INFO_LEVEL_1 *info_1;
 	SPOOL_PRINTER_INFO_LEVEL_2 *info_2;
 	SPOOL_PRINTER_INFO_LEVEL_3 *info_3;
+	SPOOL_PRINTER_INFO_LEVEL_7 *info_7;
 }
 SPOOL_PRINTER_INFO_LEVEL;
 
@@ -1493,7 +1562,7 @@ typedef struct {
 	uint32 previousnames_len;
 	uint32 previousnames_ptr;
 	NTTIME	driverdate;
-	UINT64_S	driverversion;
+	uint64	driverversion;
 	uint32	dummy4;
 	uint32 mfgname_ptr;
 	uint32 oemurl_ptr;
@@ -1526,28 +1595,6 @@ typedef struct spool_printer_driver_info_level
 SPOOL_PRINTER_DRIVER_INFO_LEVEL;
 
 
-/* this struct is undocumented */
-/* thanks to the ddk ... */
-typedef struct spool_user_level_1
-{
-	uint32 size;
-	uint32 client_name_ptr;
-	uint32 user_name_ptr;
-	uint32 build;
-	uint32 major;
-	uint32 minor;
-	uint32 processor;
-	UNISTR2 client_name;
-	UNISTR2 user_name;
-}
-SPOOL_USER_LEVEL_1;
-
-typedef struct spool_user_level
-{
-	SPOOL_USER_LEVEL_1 *user_level_1;
-}
-SPOOL_USER_LEVEL;
-
 typedef struct spool_q_setprinter
 {
 	POLICY_HND handle;
@@ -1567,70 +1614,46 @@ typedef struct spool_r_setprinter
 }
 SPOOL_R_SETPRINTER;
 
-typedef struct spool_q_addprinter
-{
-	UNISTR2 server_name;
-	uint32 level;
-	SPOOL_PRINTER_INFO_LEVEL info;
-	DEVMODE_CTR devmode_ctr;
-	SEC_DESC_BUF *secdesc_ctr;
-	uint32 user_level;
-	SPOOL_USER_LEVEL user;
-}
-SPOOL_Q_ADDPRINTER;
+/********************************************/
 
-typedef struct spool_r_addprinter
-{
-	WERROR status;
-}
-SPOOL_R_ADDPRINTER;
-
-typedef struct spool_q_deleteprinter
-{
+typedef struct {
 	POLICY_HND handle;
-}
-SPOOL_Q_DELETEPRINTER;
+} SPOOL_Q_DELETEPRINTER;
 
-typedef struct spool_r_deleteprinter
-{
+typedef struct {
 	POLICY_HND handle;
 	WERROR status;
-}
-SPOOL_R_DELETEPRINTER;
+} SPOOL_R_DELETEPRINTER;
 
-typedef struct spool_q_abortprinter
-{
+/********************************************/
+
+typedef struct {
 	POLICY_HND handle;
-}
-SPOOL_Q_ABORTPRINTER;
+} SPOOL_Q_ABORTPRINTER;
 
-typedef struct spool_r_abortprinter
-{
+typedef struct {
 	WERROR status;
-}
-SPOOL_R_ABORTPRINTER;
+} SPOOL_R_ABORTPRINTER;
 
 
-typedef struct spool_q_addprinterex
-{
-	uint32 server_name_ptr;
-	UNISTR2 server_name;
+/********************************************/
+
+typedef struct {
+	UNISTR2 *server_name;
 	uint32 level;
 	SPOOL_PRINTER_INFO_LEVEL info;
 	DEVMODE_CTR devmode_ctr;
 	SEC_DESC_BUF *secdesc_ctr;
 	uint32 user_switch;
 	SPOOL_USER_CTR user_ctr;
-}
-SPOOL_Q_ADDPRINTEREX;
+} SPOOL_Q_ADDPRINTEREX;
 
-typedef struct spool_r_addprinterex
-{
+typedef struct {
 	POLICY_HND handle;
 	WERROR status;
-}
-SPOOL_R_ADDPRINTEREX;
+} SPOOL_R_ADDPRINTEREX;
 
+/********************************************/
 
 typedef struct spool_q_addprinterdriver
 {
@@ -1647,6 +1670,22 @@ typedef struct spool_r_addprinterdriver
 }
 SPOOL_R_ADDPRINTERDRIVER;
 
+typedef struct spool_q_addprinterdriverex
+{
+	uint32 server_name_ptr;
+	UNISTR2 server_name;
+	uint32 level;
+	SPOOL_PRINTER_DRIVER_INFO_LEVEL info;
+	uint32 copy_flags;
+}
+SPOOL_Q_ADDPRINTERDRIVEREX;
+
+typedef struct spool_r_addprinterdriverex
+{
+	WERROR status;
+}
+SPOOL_R_ADDPRINTERDRIVEREX;
+
 
 typedef struct driver_directory_1
 {
@@ -1667,14 +1706,14 @@ typedef struct spool_q_getprinterdriverdirectory
 	uint32 environment_ptr;
 	UNISTR2 environment;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_GETPRINTERDRIVERDIR;
 
 typedef struct spool_r_getprinterdriverdirectory
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	WERROR status;
 }
@@ -1704,7 +1743,7 @@ typedef struct spool_q_enumprintprocessors
 	uint32 environment_ptr;
 	UNISTR2 environment;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMPRINTPROCESSORS;
@@ -1717,7 +1756,7 @@ PRINTPROCESSOR_1;
 
 typedef struct spool_r_enumprintprocessors
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 returned;
 	WERROR status;
@@ -1731,7 +1770,7 @@ typedef struct spool_q_enumprintprocdatatypes
 	uint32 processor_ptr;
 	UNISTR2 processor;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMPRINTPROCDATATYPES;
@@ -1744,7 +1783,7 @@ PRINTPROCDATATYPE_1;
 
 typedef struct spool_r_enumprintprocdatatypes
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 returned;
 	WERROR status;
@@ -1770,14 +1809,14 @@ typedef struct spool_q_enumprintmonitors
 	uint32 name_ptr;
 	UNISTR2 name;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_ENUMPRINTMONITORS;
 
 typedef struct spool_r_enumprintmonitors
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	uint32 returned;
 	WERROR status;
@@ -1905,7 +1944,7 @@ typedef struct spool_q_getjob
 	POLICY_HND handle;
 	uint32 jobid;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_GETJOB;
@@ -1925,7 +1964,7 @@ PJOB_INFO;
 
 typedef struct spool_r_getjob
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	WERROR status;
 }
@@ -2035,6 +2074,21 @@ typedef struct spool_r_setprinterdataex
 SPOOL_R_SETPRINTERDATAEX;
 
 
+typedef struct spool_q_deleteprinterdataex
+{
+	POLICY_HND handle;
+	UNISTR2 keyname;
+	UNISTR2 valuename;
+}
+SPOOL_Q_DELETEPRINTERDATAEX;
+
+typedef struct spool_r_deleteprinterdataex
+{
+	WERROR status;
+}
+SPOOL_R_DELETEPRINTERDATAEX;
+
+
 typedef struct spool_q_enumprinterkey
 {
 	POLICY_HND handle;
@@ -2051,6 +2105,19 @@ typedef struct spool_r_enumprinterkey
 }
 SPOOL_R_ENUMPRINTERKEY;
 
+typedef struct spool_q_deleteprinterkey
+{
+	POLICY_HND handle;
+	UNISTR2 keyname;
+}
+SPOOL_Q_DELETEPRINTERKEY;
+
+typedef struct spool_r_deleteprinterkey
+{
+	WERROR status;
+}
+SPOOL_R_DELETEPRINTERKEY;
+
 typedef struct printer_enum_values
 {
 	UNISTR valuename;
@@ -2098,19 +2165,59 @@ typedef struct spool_q_getprintprocessordirectory
 	UNISTR2 name;
 	UNISTR2 environment;
 	uint32 level;
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 offered;
 }
 SPOOL_Q_GETPRINTPROCESSORDIRECTORY;
 
 typedef struct spool_r_getprintprocessordirectory
 {
-	NEW_BUFFER *buffer;
+	RPC_BUFFER *buffer;
 	uint32 needed;
 	WERROR status;
 }
 SPOOL_R_GETPRINTPROCESSORDIRECTORY;
 
+/**************************************/
+
+#define MAX_PORTNAME		64
+#define MAX_NETWORK_NAME	49
+#define MAX_SNMP_COMM_NAME	33
+#define	MAX_QUEUE_NAME		33
+#define MAX_IPADDR_STRING	17
+		
+typedef struct {
+	uint16 portname[MAX_PORTNAME];
+	uint32 version;
+	uint32 protocol;
+	uint32 size;
+	uint32 reserved;
+	uint16 hostaddress[MAX_NETWORK_NAME];
+	uint16 snmpcommunity[MAX_SNMP_COMM_NAME];
+	uint32 dblspool;
+	uint16 queue[MAX_QUEUE_NAME];
+	uint16 ipaddress[MAX_IPADDR_STRING];
+	uint32 port;
+	uint32 snmpenabled;
+	uint32 snmpdevindex;
+} SPOOL_PORT_DATA_1;
+
+typedef struct {
+	POLICY_HND handle;
+	UNISTR2 dataname;
+	RPC_BUFFER indata;
+	uint32 indata_len;
+	uint32 offered;
+	uint32 unknown;
+} SPOOL_Q_XCVDATAPORT;
+
+typedef struct {
+	RPC_BUFFER outdata;
+	uint32 needed;
+	uint32 unknown;
+	WERROR status;
+} SPOOL_R_XCVDATAPORT;
+
 #define PRINTER_DRIVER_VERSION 2
 #define PRINTER_DRIVER_ARCHITECTURE "Windows NT x86"
 
