@@ -146,7 +146,7 @@ static int ms_fnmatch_core(const smb_ucs2_t *p, const smb_ucs2_t *n,
 	return -1;
 }
 
-int ms_fnmatch(const char *pattern, const char *string, enum protocol_types protocol, 
+int ms_fnmatch(const char *pattern, const char *string, BOOL translate_pattern,
 	       BOOL is_case_sensitive)
 {
 	wpstring p, s;
@@ -167,10 +167,19 @@ int ms_fnmatch(const char *pattern, const char *string, enum protocol_types prot
 		}
 	}
 
-	pstrcpy_wa(p, pattern);
-	pstrcpy_wa(s, string);
+	if (push_ucs2(NULL, p, pattern, sizeof(p), STR_TERMINATE) == (size_t)-1) {
+		/* Not quite the right answer, but finding the right one
+		  under this failure case is expensive, and it's pretty close */
+		return -1;
+	}
 
-	if (protocol <= PROTOCOL_LANMAN2) {
+	if (push_ucs2(NULL, s, string, sizeof(s), STR_TERMINATE) == (size_t)-1) {
+		/* Not quite the right answer, but finding the right one
+		   under this failure case is expensive, and it's pretty close */
+		return -1;
+	}
+
+	if (translate_pattern) {
 		/*
 		  for older negotiated protocols it is possible to
 		  translate the pattern to produce a "new style"
