@@ -32,7 +32,8 @@ pid_t pidfile_pid(const char *name)
 {
 	int fd;
 	char pidstr[20];
-	unsigned ret;
+	pid_t pid;
+	unsigned int ret;
 	pstring pidFile;
 
 	slprintf(pidFile, sizeof(pidFile)-1, "%s/%s.pid", lp_piddir(), name);
@@ -57,7 +58,8 @@ pid_t pidfile_pid(const char *name)
 		goto noproc;
 	}
 	
-	if (!process_exists((pid_t)ret)) {
+	pid = (pid_t)ret;
+	if (!process_exists_by_pid(pid)) {
 		goto noproc;
 	}
 
@@ -76,13 +78,32 @@ pid_t pidfile_pid(const char *name)
 }
 
 /* create a pid file in the pid directory. open it and leave it locked */
-void pidfile_create(const char *name)
+void pidfile_create(const char *program_name)
 {
 	int     fd;
 	char    buf[20];
+	char    *short_configfile;
+	pstring name;
 	pstring pidFile;
 	pid_t pid;
 
+	/* Add a suffix to the program name if this is a process with a
+	 * none default configuration file name. */
+	if (strcmp( CONFIGFILE, dyn_CONFIGFILE) == 0) {
+		strncpy( name, program_name, sizeof( name)-1);
+	} else {
+		short_configfile = strrchr( dyn_CONFIGFILE, '/');
+		if (short_configfile == NULL) {
+			/* conf file in current directory */
+			short_configfile = dyn_CONFIGFILE;
+		} else {
+			/* full/relative path provided */
+			short_configfile++;
+		}
+		slprintf( name, sizeof( name)-1, "%s-%s", program_name,
+			  short_configfile );
+	}
+
 	slprintf(pidFile, sizeof(pidFile)-1, "%s/%s.pid", lp_piddir(), name);
 
 	pid = pidfile_pid(name);
