@@ -774,6 +774,11 @@ static BOOL set_process_capability(enum smbd_capability capability,
 #elif CAP_MKNOD
 			/* Linux has CAP_MKNOD for DMAPI access. */
 			cap_vals[num_cap_vals++] = CAP_MKNOD;
+#endif
+			break;
+		case LEASE_CAPABILITY:
+#ifdef CAP_LEASE
+			cap_vals[num_cap_vals++] = CAP_LEASE;
 #endif
 			break;
 	}
