@@ -95,9 +95,14 @@ BOOL cli_send_trans(struct cli_state *cli, int trans,
 		return False;
 	}
 
+	/* Note we're in a trans state. Save the sequence
+	 * numbers for replies. */
+	client_set_trans_sign_state_on(cli, mid);
+
 	if (this_ldata < ldata || this_lparam < lparam) {
 		/* receive interim response */
 		if (!cli_receive_smb(cli) || cli_is_error(cli)) {
+			client_set_trans_sign_state_off(cli, mid);
 			return(False);
 		}
 
@@ -108,6 +113,9 @@ BOOL cli_send_trans(struct cli_state *cli, int trans,
 			this_lparam = MIN(lparam-tot_param,cli->max_xmit - 500); /* hack */
 			this_ldata = MIN(ldata-tot_data,cli->max_xmit - (500+this_lparam));
 
+			client_set_trans_sign_state_off(cli, mid);
+			client_set_trans_sign_state_on(cli, mid);
+
 			set_message(cli->outbuf,trans==SMBtrans?8:9,0,True);
 			SCVAL(cli->outbuf,smb_com,(trans==SMBtrans ? SMBtranss : SMBtranss2));
 			
@@ -139,6 +147,7 @@ BOOL cli_send_trans(struct cli_state *cli, int trans,
 
 			show_msg(cli->outbuf);
 			if (!cli_send_smb(cli)) {
+				client_set_trans_sign_state_off(cli, mid);
 				return False;
 			}
 
@@ -317,6 +326,7 @@ BOOL cli_receive_trans(struct cli_state *cli,int trans,
 
   out:
 
+	client_set_trans_sign_state_off(cli, SVAL(cli->inbuf,smb_mid));
 	return ret;
 }
 
@@ -384,9 +394,14 @@ BOOL cli_send_nt_trans(struct cli_state *cli,
 		return False;
 	}	
 
+	/* Note we're in a trans state. Save the sequence
+	 * numbers for replies. */
+	client_set_trans_sign_state_on(cli, mid);
+
 	if (this_ldata < ldata || this_lparam < lparam) {
 		/* receive interim response */
 		if (!cli_receive_smb(cli) || cli_is_error(cli)) {
+			client_set_trans_sign_state_off(cli, mid);
 			return(False);
 		}
 
@@ -428,6 +443,7 @@ BOOL cli_send_nt_trans(struct cli_state *cli,
 			show_msg(cli->outbuf);
 
 			if (!cli_send_smb(cli)) {
+				client_set_trans_sign_state_off(cli, mid);
 				return False;
 			}
 			
@@ -627,5 +643,6 @@ BOOL cli_receive_nt_trans(struct cli_state *cli,
 
   out:
 
+	client_set_trans_sign_state_off(cli, SVAL(cli->inbuf,smb_mid));
 	return ret;
 }
