@@ -34,6 +34,8 @@ BOOL global_in_nmbd = False;
 ****************************************************************************/
 #define SAFKEY_FMT	"SAF/DOMAIN/%s"
 #define SAF_TTL		900
+#define SAFJOINKEY_FMT	"SAFJOIN/DOMAIN/%s"
+#define SAFJOIN_TTL	3600
 
 static char *saf_key(const char *domain)
 {
@@ -44,6 +46,15 @@ static char *saf_key(const char *domain)
 	return keystr;
 }
 
+static char *saf_join_key(const char *domain)
+{
+	char *keystr;
+
+	asprintf( &keystr, SAFJOINKEY_FMT, strupper_static(domain) );
+
+	return keystr;
+}
+
 /****************************************************************************
 ****************************************************************************/
 
@@ -67,7 +78,7 @@ BOOL saf_store( const char *domain, const char *servername )
 		return False;
 	
 	key = saf_key( domain );
-	expire = time( NULL ) + SAF_TTL;
+	expire = time( NULL ) + lp_parm_int(-1, "saf","ttl", SAF_TTL);
 	
 	
 	DEBUG(10,("saf_store: domain = [%s], server = [%s], expire = [%u]\n",
@@ -80,6 +91,38 @@ BOOL saf_store( const char *domain, const char *servername )
 	return ret;
 }
 
+BOOL saf_join_store( const char *domain, const char *servername )
+{
+	char *key;
+	time_t expire;
+	BOOL ret = False;
+
+	if ( !domain || !servername ) {
+		DEBUG(2,("saf_join_store: Refusing to store empty domain or servername!\n"));
+		return False;
+	}
+
+	if ( (strlen(domain) == 0) || (strlen(servername) == 0) ) {
+		DEBUG(0,("saf_join_store: refusing to store 0 length domain or servername!\n"));
+		return False;
+	}
+
+	if ( !gencache_init() )
+		return False;
+
+	key = saf_join_key( domain );
+	expire = time( NULL ) + lp_parm_int(-1, "saf","join ttl", SAFJOIN_TTL);
+
+	DEBUG(10,("saf_join_store: domain = [%s], server = [%s], expire = [%u]\n",
+		domain, servername, (unsigned int)expire ));
+
+	ret = gencache_set( key, servername, expire );
+
+	SAFE_FREE( key );
+
+	return ret;
+}
+
 BOOL saf_delete( const char *domain )
 {
 	char *key;
@@ -93,15 +136,22 @@ BOOL saf_delete( const char *domain )
 	if ( !gencache_init() ) 
 		return False;
 	
+	key = saf_join_key(domain);
+	ret = gencache_del(key);
+	SAFE_FREE(key);
+
+	if (ret) {
+		DEBUG(10,("saf_delete[join]: domain = [%s]\n", domain ));
+	}
+
 	key = saf_key(domain);
 	ret = gencache_del(key);
-	
+	SAFE_FREE(key);
+
 	if (ret) {
 		DEBUG(10,("saf_delete: domain = [%s]\n", domain ));		
 	}
 
-	SAFE_FREE( key );
-
 	return ret;
 }
 
@@ -122,7 +172,19 @@ char *saf_fetch( const char *domain )
 	
 	if ( !gencache_init() ) 
 		return False;
-	
+
+	key = saf_join_key( domain );
+
+	ret = gencache_get( key, &server, &timeout );
+
+	SAFE_FREE( key );
+
+	if ( ret ) {
+		DEBUG(5,("saf_fetch[join]: Returning \"%s\" for \"%s\" domain\n",
+			server, domain ));
+		return server;
+	}
+
 	key = saf_key( domain );
 	
 	ret = gencache_get( key, &server, &timeout );
@@ -1163,7 +1225,7 @@ BOOL internal_resolve_name(const char *name, int name_type,
 	*return_count = 0;
 
 	DEBUG(10, ("internal_resolve_name: looking up %s#%x (sitename %s)\n",
-			name, name_type, sitename ? sitename : NULL));
+			name, name_type, sitename ? sitename : "(null)"));
 
 	if (allzeros || allones || is_address) {
   
@@ -1428,6 +1490,10 @@ static NTSTATUS get_dc_list(const char *domain, const char *sitename, struct ip_
 	struct ip_service *auto_ip_list = NULL;
 	BOOL done_auto_lookup = False;
 	int auto_count = 0;
+	NTSTATUS status;
+
+	*ip_list = NULL;
+	*count = 0;
 
 	*ordered = False;
 
@@ -1480,9 +1546,11 @@ static NTSTATUS get_dc_list(const char *domain, const char *sitename, struct ip_
 		 * NTSTATUS */
 		if (internal_resolve_name(domain, 0x1C, sitename, ip_list, count,
 					  resolve_order)) {
-			return NT_STATUS_OK;
+			status = NT_STATUS_OK;
+			goto out;
 		} else {
-			return NT_STATUS_NO_LOGON_SERVERS;
+			status = NT_STATUS_NO_LOGON_SERVERS;
+			goto out;
 		}
 	}
 
@@ -1497,7 +1565,7 @@ static NTSTATUS get_dc_list(const char *domain, const char *sitename, struct ip_
 
 	p = pserver;
 	while (next_token(&p,name,LIST_SEP,sizeof(name))) {
-		if (strequal(name, "*")) {
+		if (!done_auto_lookup && strequal(name, "*")) {
 			if (internal_resolve_name(domain, 0x1C, sitename, &auto_ip_list,
 						  &auto_count, resolve_order))
 				num_addresses += auto_count;
@@ -1514,21 +1582,23 @@ static NTSTATUS get_dc_list(const char *domain, const char *sitename, struct ip_
 	if ( (num_addresses == 0) ) {
 		if ( done_auto_lookup ) {
 			DEBUG(4,("get_dc_list: no servers found\n")); 
-			SAFE_FREE(auto_ip_list);
-			return NT_STATUS_NO_LOGON_SERVERS;
+			status = NT_STATUS_NO_LOGON_SERVERS;
+			goto out;
 		}
 		if (internal_resolve_name(domain, 0x1C, sitename, ip_list, count,
 					  resolve_order)) {
-			return NT_STATUS_OK;
+			status = NT_STATUS_OK;
+			goto out;
 		} else {
-			return NT_STATUS_NO_LOGON_SERVERS;
+			status = NT_STATUS_NO_LOGON_SERVERS;
+			goto out;
 		}
 	}
 
 	if ( (return_iplist = SMB_MALLOC_ARRAY(struct ip_service, num_addresses)) == NULL ) {
 		DEBUG(3,("get_dc_list: malloc fail !\n"));
-		SAFE_FREE(auto_ip_list);
-		return NT_STATUS_NO_MEMORY;
+		status = NT_STATUS_NO_MEMORY;
+		goto out;
 	}
 
 	p = pserver;
@@ -1585,8 +1655,6 @@ static NTSTATUS get_dc_list(const char *domain, const char *sitename, struct ip_
 		}
 	}
 				
-	SAFE_FREE(auto_ip_list);
-
 	/* need to remove duplicates in the list if we have any 
 	   explicit password servers */
 	   
@@ -1606,7 +1674,19 @@ static NTSTATUS get_dc_list(const char *domain, const char *sitename, struct ip_
 	*ip_list = return_iplist;
 	*count = local_count;
 
-	return ( *count != 0 ? NT_STATUS_OK : NT_STATUS_NO_LOGON_SERVERS );
+	status = ( *count != 0 ? NT_STATUS_OK : NT_STATUS_NO_LOGON_SERVERS );
+
+  out:
+
+	if (!NT_STATUS_IS_OK(status)) {
+		SAFE_FREE(return_iplist);
+		*ip_list = NULL;
+		*count = 0;
+	}
+
+	SAFE_FREE(auto_ip_list);
+
+	return status;
 }
 
 /*********************************************************************
@@ -1630,6 +1710,14 @@ NTSTATUS get_sorted_dc_list( const char *domain, const char *sitename, struct ip
 	}
 
 	status = get_dc_list(domain, sitename, ip_list, count, lookup_type, &ordered);
+	if (NT_STATUS_EQUAL(status, NT_STATUS_NO_LOGON_SERVERS) && sitename) {
+		DEBUG(3,("get_sorted_dc_list: no server for name %s available"
+			 " in site %s, fallback to all servers\n",
+			 domain, sitename));
+		status = get_dc_list(domain, NULL, ip_list, count,
+				     lookup_type, &ordered);
+	}
+
 	if (!NT_STATUS_IS_OK(status)) {
 		return status; 
 	}
