@@ -46,7 +46,7 @@ struct event_context *winbind_event_context(void)
 
 /* Reload configuration */
 
-static BOOL reload_services_file(void)
+static BOOL reload_services_file(const char *logfile)
 {
 	BOOL ret;
 
@@ -62,6 +62,11 @@ static BOOL reload_services_file(void)
 	reopen_logs();
 	ret = lp_load(dyn_CONFIGFILE,False,False,True,True);
 
+	/* if this is a child, restore the logfile to the special
+	   name - <domain>, idmap, etc. */
+	if (logfile && *logfile)
+		lp_set_logfile(logfile);
+
 	reopen_logs();
 	load_interfaces();
 
@@ -117,14 +122,21 @@ static void flush_caches(void)
 
 /* Handle the signal by unlinking socket and exiting */
 
-static void terminate(void)
+static void terminate(bool in_parent)
 {
-	pstring path;
+	if (in_parent) {
+		/* When parent goes away we should
+		 * remove the socket file. Not so
+		 * when children terminate.
+		 */ 
 
-	/* Remove socket file */
-	pstr_sprintf(path, "%s/%s", 
-		 WINBINDD_SOCKET_DIR, WINBINDD_SOCKET_NAME);
-	unlink(path);
+		pstring path;
+
+		/* Remove socket file */
+		pstr_sprintf(path, "%s/%s", 
+			WINBINDD_SOCKET_DIR, WINBINDD_SOCKET_NAME);
+		unlink(path);
+	}
 
 	idmap_close();
 	
@@ -181,7 +193,7 @@ static void msg_reload_services(int msg_type, struct process_id src,
 {
         /* Flush various caches */
 	flush_caches();
-	reload_services_file();
+	reload_services_file((const char *) private_data);
 }
 
 /* React on 'smbcontrol winbindd shutdown' in the same way as on SIGTERM*/
@@ -465,7 +477,6 @@ static void response_extra_sent(void *private_data, BOOL success)
 		return;
 	}
 
-	SAFE_FREE(state->request.extra_data.data);
 	SAFE_FREE(state->response.extra_data.data);
 
 	setup_async_read(&state->fd_event, &state->request, sizeof(uint32),
@@ -500,6 +511,8 @@ static void response_main_sent(void *private_data, BOOL success)
 
 static void request_finished(struct winbindd_cli_state *state)
 {
+	/* Make sure request.extra_data is freed when finish processing a request */
+	SAFE_FREE(state->request.extra_data.data);
 	setup_async_write(&state->fd_event, &state->response,
 			  sizeof(state->response), response_main_sent, state);
 }
@@ -656,12 +669,17 @@ static void new_connection(int listen_sock, BOOL privileged)
 
 static void remove_client(struct winbindd_cli_state *state)
 {
+	char c = 0;
+
 	/* It's a dead client - hold a funeral */
 	
 	if (state == NULL) {
 		return;
 	}
-		
+
+	/* tell client, we are closing ... */
+	write(state->sock, &c, sizeof(c));
+
 	/* Close socket */
 		
 	close(state->sock);
@@ -718,6 +736,25 @@ static BOOL remove_idle_client(void)
 	return False;
 }
 
+/* check if HUP has been received and reload files */
+void winbind_check_sighup(const char *logfile)
+{
+	if (do_sighup) {
+
+		DEBUG(3, ("got SIGHUP\n"));
+
+		msg_reload_services(MSG_SMB_CONF_UPDATED, pid_to_procid(0), NULL, 0, logfile);
+		do_sighup = False;
+	}
+}
+
+/* check if TERM has been received */
+void winbind_check_sigterm(bool in_parent)
+{
+	if (do_sigterm)
+		terminate(in_parent);
+}
+
 /* Process incoming clients on listen_sock.  We use a tricky non-blocking,
    non-forking, non-threaded model which allows us to handle many
    simultaneous connections while remaining impervious to many denial of
@@ -882,16 +919,8 @@ static void process_loop(void)
 
 	/* Check signal handling things */
 
-	if (do_sigterm)
-		terminate();
-
-	if (do_sighup) {
-
-		DEBUG(3, ("got SIGHUP\n"));
-
-		msg_reload_services(MSG_SMB_CONF_UPDATED, pid_to_procid(0), NULL, 0, NULL);
-		do_sighup = False;
-	}
+	winbind_check_sigterm(true);
+	winbind_check_sighup(NULL);
 
 	if (do_sigusr2) {
 		print_winbindd_status();
@@ -989,7 +1018,7 @@ int main(int argc, char **argv, char **envp)
 		  SAMBA_VERSION_STRING, 
 		  COPYRIGHT_STARTUP_MESSAGE) );
 
-	if (!reload_services_file()) {
+	if (!reload_services_file(NULL)) {
 		DEBUG(0, ("error opening config file\n"));
 		exit(1);
 	}
