@@ -37,6 +37,7 @@ extern BOOL opt_nocache;
 #ifdef HAVE_ADS
 extern struct winbindd_methods ads_methods;
 #endif
+extern struct winbindd_methods builtin_passdb_methods;
 
 /*
  * JRA. KEEP THIS LIST UP TO DATE IF YOU ADD CACHE ENTRIES.
@@ -136,6 +137,10 @@ static struct winbind_cache *get_cache(struct winbindd_domain *domain)
 
 	/* We have to know what type of domain we are dealing with first. */
 
+	if (domain->internal) {
+		domain->backend = &builtin_passdb_methods;
+		domain->initialized = True;
+	}
 	if ( !domain->initialized ) {
 		init_dc_connection( domain );
 	}
@@ -1133,14 +1138,15 @@ do_query:
 			domain->name ));
 
 		status = domain->backend->query_user_list(domain, mem_ctx, num_entries, info);
-		if (!NT_STATUS_IS_OK(status))
+		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(3, ("query_user_list: returned 0x%08x, "
 				  "retrying\n", NT_STATUS_V(status)));
-			if (NT_STATUS_EQUAL(status, NT_STATUS_UNSUCCESSFUL)) {
-				DEBUG(3, ("query_user_list: flushing "
-					  "connection cache\n"));
-				invalidate_cm_connection(&domain->conn);
-			}
+		}
+		if (NT_STATUS_EQUAL(status, NT_STATUS_UNSUCCESSFUL)) {
+			DEBUG(3, ("query_user_list: flushing "
+				  "connection cache\n"));
+			invalidate_cm_connection(&domain->conn);
+		}
 
 	} while (NT_STATUS_V(status) == NT_STATUS_V(NT_STATUS_UNSUCCESSFUL) && 
 		 (retry++ < 5));
@@ -2116,7 +2122,9 @@ do_query:
 
 	/* and save it */
 	refresh_sequence_number(domain, False);
-	wcache_save_password_policy(domain, status, policy);
+	if (NT_STATUS_IS_OK(status)) {
+		wcache_save_password_policy(domain, status, policy);
+	}
 
 	return status;
 }
