@@ -25,6 +25,7 @@
 #include <cups/cups.h>
 #include <cups/language.h>
 
+extern userdom_struct current_user_info;
 
 /*
  * 'cups_passwd_cb()' - The CUPS password callback...
@@ -40,16 +41,38 @@ cups_passwd_cb(const char *prompt)	/* I - Prompt */
 	return (NULL);
 }
 
-static const char *cups_server(void)
+static http_t *cups_connect(void)
 {
-	if ((lp_cups_server() != NULL) && (strlen(lp_cups_server()) > 0)) {
-		DEBUG(10, ("cups server explicitly set to %s\n",
-			   lp_cups_server()));
-		return lp_cups_server();
+	http_t *http;
+	char *server, *p;
+	int port;
+	
+	if (lp_cups_server() != NULL && strlen(lp_cups_server()) > 0) {
+		server = smb_xstrdup(lp_cups_server());
+	} else {
+		server = smb_xstrdup(cupsServer());
 	}
 
-	DEBUG(10, ("cups server left to default %s\n", cupsServer()));
-	return cupsServer();
+	p = strchr(server, ':');
+	if (p) {
+		port = atoi(p+1);
+		*p = '\0';
+	} else {
+		port = ippPort();
+	}
+	
+	DEBUG(10, ("connecting to cups server %s:%d\n",
+		   server, port));
+
+	if ((http = httpConnect(server, port)) == NULL) {
+		DEBUG(0,("Unable to connect to CUPS server %s:%d - %s\n", 
+			 server, port, strerror(errno)));
+		SAFE_FREE(server);
+		return NULL;
+	}
+
+	SAFE_FREE(server);
+	return http;
 }
 
 BOOL cups_cache_reload(void)
@@ -80,9 +103,7 @@ BOOL cups_cache_reload(void)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -265,7 +286,7 @@ BOOL cups_cache_reload(void)
  * 'cups_job_delete()' - Delete a job.
  */
 
-static int cups_job_delete(int snum, struct printjob *pjob)
+static int cups_job_delete(const char *sharename, const char *lprm_command, struct printjob *pjob)
 {
 	int		ret = 1;		/* Return value */
 	http_t		*http = NULL;		/* HTTP connection to server */
@@ -275,7 +296,7 @@ static int cups_job_delete(int snum, struct printjob *pjob)
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 
 
-	DEBUG(5,("cups_job_delete(%d, %p (%d))\n", snum, pjob, pjob->sysjob));
+	DEBUG(5,("cups_job_delete(%s, %p (%d))\n", sharename, pjob, pjob->sysjob));
 
        /*
         * Make sure we don't ask for passwords...
@@ -287,9 +308,7 @@ static int cups_job_delete(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -379,9 +398,7 @@ static int cups_job_pause(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -471,9 +488,7 @@ static int cups_job_resume(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -566,9 +581,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -712,7 +725,7 @@ static int cups_queue_get(const char *sharename,
 
 	*q = NULL;
 
-	/* HACK ALERT!!!  The porblem with support the 'printer name' 
+	/* HACK ALERT!!!  The problem with support the 'printer name' 
 	   option is that we key the tdb off the sharename.  So we will 
 	   overload the lpq_command string to pass in the printername 
 	   (which is basically what we do for non-cups printers ... using 
@@ -732,9 +745,7 @@ static int cups_queue_get(const char *sharename,
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -816,16 +827,13 @@ static int cups_queue_get(const char *sharename,
 		if (qcount >= qalloc) {
 			qalloc += 16;
 
-			temp = SMB_REALLOC_ARRAY(queue, print_queue_struct, qalloc);
+			queue = SMB_REALLOC_ARRAY(queue, print_queue_struct, qalloc);
 
-			if (temp == NULL) {
+			if (queue == NULL) {
 				DEBUG(0,("cups_queue_get: Not enough memory!"));
 				qcount = 0;
-				SAFE_FREE(queue);
 				goto out;
 			}
-
-			queue = temp;
 		}
 
 		temp = queue + qcount;
@@ -999,7 +1007,6 @@ static int cups_queue_get(const char *sharename,
 
 static int cups_queue_pause(int snum)
 {
-	extern userdom_struct current_user_info;
 	int		ret = 1;		/* Return value */
 	http_t		*http = NULL;		/* HTTP connection to server */
 	ipp_t		*request = NULL,	/* IPP Request */
@@ -1020,9 +1027,7 @@ static int cups_queue_pause(int snum)
 	 * Try to connect to the server...
 	 */
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -1093,7 +1098,6 @@ static int cups_queue_pause(int snum)
 
 static int cups_queue_resume(int snum)
 {
-	extern userdom_struct current_user_info;
 	int		ret = 1;		/* Return value */
 	http_t		*http = NULL;		/* HTTP connection to server */
 	ipp_t		*request = NULL,	/* IPP Request */
@@ -1114,9 +1118,7 @@ static int cups_queue_resume(int snum)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
-		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cups_server(), strerror(errno)));
+	if ((http = cups_connect()) == NULL) {
 		goto out;
 	}
 
@@ -1196,7 +1198,144 @@ struct printif	cups_printif =
 	cups_job_submit,
 };
 
+BOOL cups_pull_comment_location(NT_PRINTER_INFO_LEVEL_2 *printer)
+{
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	ipp_attribute_t	*attr;		/* Current attribute */
+	cups_lang_t	*language = NULL;	/* Default language */
+	char		*name,		/* printer-name attribute */
+			*info,		/* printer-info attribute */
+			*location;	/* printer-location attribute */
+	char		uri[HTTP_MAX_URI];
+	static const char *requested[] =/* Requested attributes */
+			{
+			  "printer-name",
+			  "printer-info",
+			  "printer-location"
+			};
+	BOOL ret = False;
+
+	DEBUG(5, ("pulling %s location\n", printer->sharename));
+
+	/*
+	 * Make sure we don't ask for passwords...
+	 */
+
+        cupsSetPasswordCB(cups_passwd_cb);
+
+	/*
+	 * Try to connect to the server...
+	 */
+
+	if ((http = cups_connect()) == NULL) {
+		goto out;
+	}
+
+	request = ippNew();
+
+	request->request.op.operation_id = IPP_GET_PRINTER_ATTRIBUTES;
+	request->request.op.request_id   = 1;
+
+	language = cupsLangDefault();
+
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
+                     "attributes-charset", NULL, cupsLangEncoding(language));
+
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
+                     "attributes-natural-language", NULL, language->language);
+
+	slprintf(uri, sizeof(uri) - 1, "ipp://%s/printers/%s",
+		 lp_cups_server(), printer->sharename);
+
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+                     "printer-uri", NULL, uri);
+
+        ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+	              "requested-attributes",
+		      (sizeof(requested) / sizeof(requested[0])),
+		      NULL, requested);
+
+	/*
+	 * Do the request and get back a response...
+	 */
+
+	if ((response = cupsDoRequest(http, request, "/")) == NULL) {
+		DEBUG(0,("Unable to get printer attributes - %s\n",
+			 ippErrorString(cupsLastError())));
+		goto out;
+	}
+
+	for (attr = response->attrs; attr != NULL;) {
+		/*
+		 * Skip leading attributes until we hit a printer...
+		 */
+
+		while (attr != NULL && attr->group_tag != IPP_TAG_PRINTER)
+			attr = attr->next;
+
+		if (attr == NULL)
+        		break;
+
+		/*
+		 * Pull the needed attributes from this printer...
+		 */
+
+		name       = NULL;
+		info       = NULL;
+		location   = NULL;
+
+		while ( attr && (attr->group_tag == IPP_TAG_PRINTER) ) {
+			/* Grab the comment if we don't have one */
+        		if ( (strcmp(attr->name, "printer-info") == 0)
+			     && (attr->value_tag == IPP_TAG_TEXT)
+			     && !strlen(printer->comment) ) 
+			{
+				DEBUG(5,("cups_pull_comment_location: Using cups comment: %s\n",
+					 attr->values[0].string.text));				
+			    	pstrcpy(printer->comment,attr->values[0].string.text);
+			}
+
+			/* Grab the location if we don't have one */ 
+			if ( (strcmp(attr->name, "printer-location") == 0)
+			     && (attr->value_tag == IPP_TAG_TEXT) 
+			     && !strlen(printer->location) )
+			{
+				DEBUG(5,("cups_pull_comment_location: Using cups location: %s\n",
+					 attr->values[0].string.text));				
+			    	fstrcpy(printer->location,attr->values[0].string.text);
+			}
+
+        		attr = attr->next;
+		}
+
+		/*
+		 * See if we have everything needed...
+		 */
+
+		if (name == NULL)
+			break;
+
+	}
+
+	ret = True;
+
+ out:
+	if (response)
+		ippDelete(response);
+
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
+}
+
 #else
  /* this keeps fussy compilers happy */
+ void print_cups_dummy(void);
  void print_cups_dummy(void) {}
 #endif /* HAVE_CUPS */
