@@ -2397,8 +2397,10 @@ uint32 print_job_start(struct current_user *user, int snum, char *jobname, NT_DE
 
 	if ((vuser = get_valid_user_struct(user->vuid)) != NULL) {
 		fstrcpy(pjob.user, lp_printjob_username(snum));
-		standard_sub_basic(vuser->user.smb_name, vuser->user.domain, 
-				   pjob.user, sizeof(pjob.user)-1);
+		standard_sub_advanced(sharename, vuser->user.smb_name, path, 
+					vuser->gid, vuser->user.smb_name, 
+					vuser->user.domain, pjob.user, 
+					sizeof(pjob.user) - 1);
 		/* ensure NULL termination */ 
 		pjob.user[sizeof(pjob.user)-1] = '\0'; 
 	} else {
@@ -2525,7 +2527,7 @@ BOOL print_job_end(int snum, uint32 jobid, enum file_close_type close_type)
 	if (ret)
 		goto fail;
 
-	/* The print job has been sucessfully handed over to the back-end */
+	/* The print job has been successfully handed over to the back-end */
 	
 	pjob->spooled = True;
 	pjob->status = LPQ_QUEUED;
@@ -2539,7 +2541,7 @@ BOOL print_job_end(int snum, uint32 jobid, enum file_close_type close_type)
 
 fail:
 
-	/* The print job was not succesfully started. Cleanup */
+	/* The print job was not successfully started. Cleanup */
 	/* Still need to add proper error return propagation! 010122:JRR */
 	unlink(pjob->filename);
 	pjob_delete(sharename, jobid);
