@@ -26,12 +26,13 @@
 #define DBGC_CLASS DBGC_RPC_SRV
 
 static TDB_CONTEXT *tdb_reg;
+static int tdb_refcount;
 
 #define VALUE_PREFIX	"SAMBA_REGVAL"
 
 /* List the deepest path into the registry.  All part components will be created.*/
 
-/* If you want to have a part of the path controlled by the tdb abd part by
+/* If you want to have a part of the path controlled by the tdb and part by
    a virtual registry db (e.g. printing), then you have to list the deepest path.
    For example,"HKLM/SOFTWARE/Microsoft/Windows NT/CurrentVersion/Print" 
    allows the reg_db backend to handle everything up to 
@@ -49,6 +50,7 @@ static const char *builtin_registry_paths[] = {
 	"HKLM\\SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Perflib\\009",
 	"HKLM\\SYSTEM\\CurrentControlSet\\Control\\Print\\Monitors",
 	"HKLM\\SYSTEM\\CurrentControlSet\\Control\\ProductOptions",
+	"HKLM\\SYSTEM\\CurrentControlSet\\Control\\Terminal Server\\DefaultUserConfiguration",
 	"HKLM\\SYSTEM\\CurrentControlSet\\Services\\TcpIp\\Parameters",
 	"HKLM\\SYSTEM\\CurrentControlSet\\Services\\Netlogon\\Parameters",
 	"HKU",
@@ -89,11 +91,25 @@ static BOOL init_registry_data( void )
 {
 	pstring path, base, remaining;
 	fstring keyname, subkeyname;
-	REGSUBKEY_CTR	subkeys;
-	REGVAL_CTR values;
+	REGSUBKEY_CTR *subkeys;
+	REGVAL_CTR *values;
 	int i;
 	const char *p, *p2;
 	UNISTR2 data;
+
+	/*
+	 * There are potentially quite a few store operations which are all
+	 * indiviually wrapped in tdb transactions. Wrapping them in a single
+	 * transaction gives just a single transaction_commit() to actually do
+	 * its fsync()s. See tdb/common/transaction.c for info about nested
+	 * transaction behaviour.
+	 */
+
+	if ( tdb_transaction_start( tdb_reg ) == -1 ) {
+		DEBUG(0, ("init_registry_data: tdb_transaction_start "
+			  "failed\n"));
+		return False;
+	}
 	
 	/* loop over all of the predefined paths and add each component */
 	
@@ -131,27 +147,38 @@ static BOOL init_registry_data( void )
 			   we are about to update the record.  We just want any 
 			   subkeys already present */
 			
-			regsubkey_ctr_init( &subkeys );
-						   
-			regdb_fetch_keys( base, &subkeys );
+			if ( !(subkeys = TALLOC_ZERO_P( NULL, REGSUBKEY_CTR )) ) {
+				DEBUG(0,("talloc() failure!\n"));
+				goto fail;
+			}
+
+			regdb_fetch_keys( base, subkeys );
 			if ( *subkeyname ) 
-				regsubkey_ctr_addkey( &subkeys, subkeyname );
-			if ( !regdb_store_keys( base, &subkeys ))
-				return False;
+				regsubkey_ctr_addkey( subkeys, subkeyname );
+			if ( !regdb_store_keys( base, subkeys ))
+				goto fail;
 			
-			regsubkey_ctr_destroy( &subkeys );
+			TALLOC_FREE( subkeys );
 		}
 	}
 
 	/* loop over all of the predefined values and add each component */
 	
 	for ( i=0; builtin_registry_values[i].path != NULL; i++ ) {
-		regval_ctr_init( &values );
-		
-		regdb_fetch_values( builtin_registry_values[i].path, &values );
-		switch( builtin_registry_values[i].type ) {
+		if ( !(values = TALLOC_ZERO_P( NULL, REGVAL_CTR )) ) {
+			DEBUG(0,("talloc() failure!\n"));
+			goto fail;
+		}
+
+		regdb_fetch_values( builtin_registry_values[i].path, values );
+
+		/* preserve existing values across restarts.  Only add new ones */
+
+		if ( !regval_ctr_key_exists( values, builtin_registry_values[i].valuename ) ) 
+		{
+			switch( builtin_registry_values[i].type ) {
 			case REG_DWORD:
-				regval_ctr_addvalue( &values, 
+				regval_ctr_addvalue( values, 
 				                     builtin_registry_values[i].valuename,
 						     REG_DWORD,
 						     (char*)&builtin_registry_values[i].data.dw_value,
@@ -160,7 +187,7 @@ static BOOL init_registry_data( void )
 				
 			case REG_SZ:
 				init_unistr2( &data, builtin_registry_values[i].data.string, UNI_STR_TERMINATE);
-				regval_ctr_addvalue( &values, 
+				regval_ctr_addvalue( values, 
 				                     builtin_registry_values[i].valuename,
 						     REG_SZ,
 						     (char*)data.buffer,
@@ -170,20 +197,36 @@ static BOOL init_registry_data( void )
 			default:
 				DEBUG(0,("init_registry_data: invalid value type in builtin_registry_values [%d]\n",
 					builtin_registry_values[i].type));
+			}
+			regdb_store_values( builtin_registry_values[i].path, values );
 		}
-		regdb_store_values( builtin_registry_values[i].path, &values );
 		
-		regval_ctr_destroy( &values );
+		TALLOC_FREE( values );
 	}
 	
+	if (tdb_transaction_commit( tdb_reg ) == -1) {
+		DEBUG(0, ("init_registry_data: Could not commit "
+			  "transaction\n"));
+		return False;
+	}
+
 	return True;
+
+ fail:
+
+	if (tdb_transaction_cancel( tdb_reg ) == -1) {
+		smb_panic("init_registry_data: tdb_transaction_cancel "
+			  "failed\n");
+	}
+
+	return False;
 }
 
 /***********************************************************************
  Open the registry database
  ***********************************************************************/
  
-BOOL init_registry_db( void )
+BOOL regdb_init( void )
 {
 	const char *vstring = "INFO/version";
 	uint32 vers_id;
@@ -191,36 +234,90 @@ BOOL init_registry_db( void )
 	if ( tdb_reg )
 		return True;
 
-	/* placeholder tdb; reinit upon startup */
-	
 	if ( !(tdb_reg = tdb_open_log(lock_path("registry.tdb"), 0, TDB_DEFAULT, O_RDWR, 0600)) )
 	{
 		tdb_reg = tdb_open_log(lock_path("registry.tdb"), 0, TDB_DEFAULT, O_RDWR|O_CREAT, 0600);
 		if ( !tdb_reg ) {
-			DEBUG(0,("init_registry: Failed to open registry %s (%s)\n",
+			DEBUG(0,("regdb_init: Failed to open registry %s (%s)\n",
 				lock_path("registry.tdb"), strerror(errno) ));
 			return False;
 		}
 		
-		DEBUG(10,("init_registry: Successfully created registry tdb\n"));
+		DEBUG(10,("regdb_init: Successfully created registry tdb\n"));
 	}
+
+	tdb_refcount = 1;
 		
 
 	vers_id = tdb_fetch_int32(tdb_reg, vstring);
 
 	if ( vers_id != REGVER_V1 ) {
+		/* any upgrade code here if needed */
+	}
 
-		/* create the registry here */
+	/* always setup the necessary keys and values */
 
-		if ( !init_registry_data() ) {
-			DEBUG(0,("init_registry: Failed to initiailize data in registry!\n"));
-			return False;
-		}
+	if ( !init_registry_data() ) {
+		DEBUG(0,("init_registry: Failed to initialize data in registry!\n"));
+		return False;
 	}
 
 	return True;
 }
 
+/***********************************************************************
+ Open the registry.  Must already have been initialized by regdb_init()
+ ***********************************************************************/
+
+WERROR regdb_open( void )
+{
+	WERROR result = WERR_OK;
+
+	if ( tdb_reg ) {
+		DEBUG(10,("regdb_open: incrementing refcount (%d)\n", tdb_refcount));
+		tdb_refcount++;
+		return WERR_OK;
+	}
+	
+	become_root();
+
+	tdb_reg = tdb_open_log(lock_path("registry.tdb"), 0, TDB_DEFAULT, O_RDWR, 0600);
+	if ( !tdb_reg ) {
+		result = ntstatus_to_werror( map_nt_error_from_unix( errno ) );
+		DEBUG(0,("regdb_open: Failed to open %s! (%s)\n", 
+			lock_path("registry.tdb"), strerror(errno) ));
+	}
+
+	unbecome_root();
+
+	tdb_refcount = 1;
+	DEBUG(10,("regdb_open: refcount reset (%d)\n", tdb_refcount));
+
+	return result;
+}
+
+/***********************************************************************
+ ***********************************************************************/
+
+int regdb_close( void )
+{
+	int ret;
+
+	tdb_refcount--;
+
+	DEBUG(10,("regdb_close: decrementing refcount (%d)\n", tdb_refcount));
+
+	if ( tdb_refcount > 0 )
+		return 0;
+
+	SMB_ASSERT( tdb_refcount >= 0 );
+
+	ret = tdb_close( tdb_reg );
+	tdb_reg = NULL;
+
+	return ret;
+}
+
 /***********************************************************************
  Add subkey strings to the registry tdb under a defined key
  fmt is the same format as tdb_pack except this function only supports
@@ -230,7 +327,7 @@ BOOL init_registry_db( void )
 static BOOL regdb_store_keys_internal( const char *key, REGSUBKEY_CTR *ctr )
 {
 	TDB_DATA kbuf, dbuf;
-	char *buffer, *tmpbuf;
+	char *buffer;
 	int i = 0;
 	uint32 len, buflen;
 	BOOL ret = True;
@@ -245,7 +342,9 @@ static BOOL regdb_store_keys_internal( const char *key, REGSUBKEY_CTR *ctr )
 
 	/* allocate some initial memory */
 		
-	buffer = SMB_MALLOC(sizeof(pstring));
+	if (!(buffer = (char *)SMB_MALLOC(sizeof(pstring)))) {
+		return False;
+	}
 	buflen = sizeof(pstring);
 	len = 0;
 	
@@ -259,12 +358,11 @@ static BOOL regdb_store_keys_internal( const char *key, REGSUBKEY_CTR *ctr )
 		len += tdb_pack( buffer+len, buflen-len, "f", regsubkey_ctr_specific_key(ctr, i) );
 		if ( len > buflen ) {
 			/* allocate some extra space */
-			if ((tmpbuf = SMB_REALLOC( buffer, len*2 )) == NULL) {
+			if ((buffer = (char *)SMB_REALLOC( buffer, len*2 )) == NULL) {
 				DEBUG(0,("regdb_store_keys: Failed to realloc memory of size [%d]\n", len*2));
 				ret = False;
 				goto done;
 			}
-			buffer = tmpbuf;
 			buflen = len*2;
 					
 			len = tdb_pack( buffer+len, buflen-len, "f", regsubkey_ctr_specific_key(ctr, i) );
@@ -297,13 +395,17 @@ BOOL regdb_store_keys( const char *key, REGSUBKEY_CTR *ctr )
 {
 	int num_subkeys, i;
 	pstring path;
-	REGSUBKEY_CTR subkeys, old_subkeys;
+	REGSUBKEY_CTR *subkeys, *old_subkeys;
 	char *oldkeyname;
 	
 	/* fetch a list of the old subkeys so we can determine if any were deleted */
 	
-	regsubkey_ctr_init( &old_subkeys );
-	regdb_fetch_keys( key, &old_subkeys );
+	if ( !(old_subkeys = TALLOC_ZERO_P( ctr, REGSUBKEY_CTR )) ) {
+		DEBUG(0,("regdb_store_keys: talloc() failure!\n"));
+		return False;
+	}
+
+	regdb_fetch_keys( key, old_subkeys );
 	
 	/* store the subkey list for the parent */
 	
@@ -314,9 +416,9 @@ BOOL regdb_store_keys( const char *key, REGSUBKEY_CTR *ctr )
 	
 	/* now delete removed keys */
 	
-	num_subkeys = regsubkey_ctr_numkeys( &old_subkeys );
+	num_subkeys = regsubkey_ctr_numkeys( old_subkeys );
 	for ( i=0; i<num_subkeys; i++ ) {
-		oldkeyname = regsubkey_ctr_specific_key( &old_subkeys, i );
+		oldkeyname = regsubkey_ctr_specific_key( old_subkeys, i );
 		if ( !regsubkey_ctr_key_exists( ctr, oldkeyname ) ) {
 			pstr_sprintf( path, "%s%c%s", key, '/', oldkeyname );
 			normalize_reg_path( path );
@@ -324,23 +426,29 @@ BOOL regdb_store_keys( const char *key, REGSUBKEY_CTR *ctr )
 		}
 	}
 
-	regsubkey_ctr_destroy( &old_subkeys );
+	TALLOC_FREE( old_subkeys );
 	
 	/* now create records for any subkeys that don't already exist */
 	
 	num_subkeys = regsubkey_ctr_numkeys( ctr );
 	for ( i=0; i<num_subkeys; i++ ) {
 		pstr_sprintf( path, "%s%c%s", key, '/', regsubkey_ctr_specific_key( ctr, i ) );
-		regsubkey_ctr_init( &subkeys );
-		if ( regdb_fetch_keys( path, &subkeys ) == -1 ) {
+
+		if ( !(subkeys = TALLOC_ZERO_P( ctr, REGSUBKEY_CTR )) ) {
+			DEBUG(0,("regdb_store_keys: talloc() failure!\n"));
+			return False;
+		}
+
+		if ( regdb_fetch_keys( path, subkeys ) == -1 ) {
 			/* create a record with 0 subkeys */
-			if ( !regdb_store_keys_internal( path, &subkeys ) ) {
+			if ( !regdb_store_keys_internal( path, subkeys ) ) {
 				DEBUG(0,("regdb_store_keys: Failed to store new record for key [%s}\n", path ));
-				regsubkey_ctr_destroy( &subkeys );
+				TALLOC_FREE( subkeys );
 				return False;
 			}
 		}
-		regsubkey_ctr_destroy( &subkeys );
+
+		TALLOC_FREE( subkeys );
 	}
 	
 	return True;
@@ -448,11 +556,13 @@ static int regdb_pack_values(REGVAL_CTR *values, char *buf, int buflen)
 	int 		len = 0;
 	int 		i;
 	REGISTRY_VALUE	*val;
-	int		num_values = regval_ctr_numvals( values );
+	int		num_values;
 
 	if ( !values )
 		return 0;
 
+	num_values = regval_ctr_numvals( values );
+
 	/* pack the number of values first */
 	
 	len += tdb_pack( buf+len, buflen-len, "d", num_values );
@@ -480,7 +590,6 @@ int regdb_fetch_values( const char* key, REGVAL_CTR *values )
 {
 	TDB_DATA data;
 	pstring keystr;
-	int len;
 
 	DEBUG(10,("regdb_fetch_values: Looking for value of key [%s] \n", key));
 	
@@ -494,7 +603,7 @@ int regdb_fetch_values( const char* key, REGVAL_CTR *values )
 		return 0;
 	}
 	
-	len = regdb_unpack_values( values, data.dptr, data.dsize );
+	regdb_unpack_values( values, data.dptr, data.dsize );
 	
 	SAFE_FREE( data.dptr );
 	
