@@ -41,7 +41,7 @@ static BOOL svcctl_io_service_status( const char *desc, SERVICE_STATUS *status,
 	if(!prs_uint32("controls_accepted", ps, depth, &status->controls_accepted))
 		return False;
 
-	if(!prs_uint32("win32_exit_code", ps, depth, &status->win32_exit_code))
+	if(!prs_werror("win32_exit_code", ps, depth, &status->win32_exit_code))
 		return False;
 
 	if(!prs_uint32("service_exit_code", ps, depth, &status->service_exit_code))
@@ -100,21 +100,6 @@ static BOOL svcctl_io_service_config( const char *desc, SERVICE_CONFIG *config,
 
 	return True;
 }
-/*******************************************************************
-********************************************************************/
-
-BOOL svcctl_io_service_description( const char *desc, UNISTR2 *svcdesc, prs_struct *ps, int depth )
-{
-
-	prs_debug(ps, depth, desc, "svcctl_io_service_description");
-	depth++;
-
-	if (!prs_io_unistr2("", ps, depth, svcdesc))
-		return False;
-
-	return True;
-}
-
 
 /*******************************************************************
 ********************************************************************/
@@ -140,6 +125,29 @@ BOOL svcctl_io_enum_services_status( const char *desc, ENUM_SERVICES_STATUS *enu
 /*******************************************************************
 ********************************************************************/
 
+BOOL svcctl_io_service_status_process( const char *desc, SERVICE_STATUS_PROCESS *status, RPC_BUFFER *buffer, int depth )
+{
+	prs_struct *ps=&buffer->prs;
+
+	prs_debug(ps, depth, desc, "svcctl_io_service_status_process");
+	depth++;
+
+	if ( !svcctl_io_service_status("status", &status->status, ps, depth) )
+		return False;
+	if(!prs_align(ps))
+		return False;
+
+	if(!prs_uint32("process_id", ps, depth, &status->process_id))
+		return False;
+	if(!prs_uint32("service_flags", ps, depth, &status->service_flags))
+		return False;
+
+	return True;
+}
+
+/*******************************************************************
+********************************************************************/
+
 uint32 svcctl_sizeof_enum_services_status( ENUM_SERVICES_STATUS *status )
 {
 	uint32 size = 0;
@@ -151,6 +159,45 @@ uint32 svcctl_sizeof_enum_services_status( ENUM_SERVICES_STATUS *status )
 	return size;
 }
 
+/********************************************************************
+********************************************************************/
+
+static uint32 sizeof_unistr2( UNISTR2 *string )
+{
+	uint32 size = 0;
+
+	if ( !string ) 
+		return 0;	
+
+	size  = sizeof(uint32) * 3;		/* length fields */
+	size += 2 * string->uni_max_len;	/* string data */
+	size += size % 4;			/* alignment */
+
+	return size;
+}
+
+/********************************************************************
+********************************************************************/
+
+uint32 svcctl_sizeof_service_config( SERVICE_CONFIG *config )
+{
+	uint32 size = 0;
+
+	size = sizeof(uint32) * 4;	/* static uint32 fields */
+
+	/* now add the UNISTR2 + pointer sizes */
+
+	size += sizeof(uint32) * sizeof_unistr2(config->executablepath);
+	size += sizeof(uint32) * sizeof_unistr2(config->loadordergroup);
+	size += sizeof(uint32) * sizeof_unistr2(config->dependencies);
+	size += sizeof(uint32) * sizeof_unistr2(config->startname);
+	size += sizeof(uint32) * sizeof_unistr2(config->displayname);
+	
+	return size;
+}
+
+
+
 /*******************************************************************
 ********************************************************************/
 
@@ -210,12 +257,12 @@ BOOL svcctl_io_q_open_scmanager(const char *desc, SVCCTL_Q_OPEN_SCMANAGER *q_u,
 	if(!prs_align(ps))
 		return False;
 
-	if(!prs_pointer("servername", ps, depth, (void**)&q_u->servername, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2))
+	if(!prs_pointer("servername", ps, depth, (void*)&q_u->servername, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2))
 		return False;
 	if(!prs_align(ps))
 		return False;
 
-	if(!prs_pointer("database", ps, depth, (void**)&q_u->database, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2))
+	if(!prs_pointer("database", ps, depth, (void*)&q_u->database, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2))
 		return False;
 	if(!prs_align(ps))
 		return False;
@@ -439,7 +486,7 @@ BOOL svcctl_io_q_enum_services_status(const char *desc, SVCCTL_Q_ENUM_SERVICES_S
 	if(!prs_uint32("buffer_size", ps, depth, &q_u->buffer_size))
 		return False;
 
-	if(!prs_pointer("resume", ps, depth, (void**)&q_u->resume, sizeof(uint32), (PRS_POINTER_CAST)prs_uint32))
+	if(!prs_pointer("resume", ps, depth, (void*)&q_u->resume, sizeof(uint32), (PRS_POINTER_CAST)prs_uint32))
 		return False;
 	
 	return True;
@@ -470,7 +517,7 @@ BOOL svcctl_io_r_enum_services_status(const char *desc, SVCCTL_R_ENUM_SERVICES_S
 	if(!prs_uint32("returned", ps, depth, &r_u->returned))
 		return False;
 
-	if(!prs_pointer("resume", ps, depth, (void**)&r_u->resume, sizeof(uint32), (PRS_POINTER_CAST)prs_uint32))
+	if(!prs_pointer("resume", ps, depth, (void*)&r_u->resume, sizeof(uint32), (PRS_POINTER_CAST)prs_uint32))
 		return False;
 
 	if(!prs_werror("status", ps, depth, &r_u->status))
@@ -499,7 +546,7 @@ BOOL svcctl_io_q_start_service(const char *desc, SVCCTL_Q_START_SERVICE *q_u, pr
 	if(!prs_uint32("parmcount", ps, depth, &q_u->parmcount))
 		return False;
 
-	if ( !prs_pointer("rights", ps, depth, (void**)&q_u->parameters, sizeof(UNISTR4_ARRAY), (PRS_POINTER_CAST)prs_unistr4_array) )
+	if ( !prs_pointer("rights", ps, depth, (void*)&q_u->parameters, sizeof(UNISTR4_ARRAY), (PRS_POINTER_CAST)prs_unistr4_array) )
 		return False;
 
 	return True;
@@ -694,7 +741,7 @@ BOOL svcctl_io_q_query_service_config2(const char *desc, SVCCTL_Q_QUERY_SERVICE_
 	if(!smb_io_pol_hnd("service_pol", &q_u->handle, ps, depth))
 		return False;
 
-	if(!prs_uint32("info_level", ps, depth, &q_u->info_level))
+	if(!prs_uint32("level", ps, depth, &q_u->level))
 		return False;
 
 	if(!prs_uint32("buffer_size", ps, depth, &q_u->buffer_size))
@@ -705,36 +752,149 @@ BOOL svcctl_io_q_query_service_config2(const char *desc, SVCCTL_Q_QUERY_SERVICE_
 
 
 /*******************************************************************
- Creates a service description response buffer.
- The format seems to be DWORD:length of buffer
-                        DWORD:offset (fixed as four)
-                        UNISTR: unicode description in the rest of the buffer
 ********************************************************************/
 
-void init_service_description_buffer(RPC_DATA_BLOB *str,  const char *service_desc, int blob_length)
+void init_service_description_buffer(SERVICE_DESCRIPTION *desc, const char *service_desc )
 {
-	uint32 offset;
-	uint8 *bp;
+	desc->unknown = 0x04;	/* always 0x0000 0004 (no idea what this is) */
+	init_unistr( &desc->description, service_desc );
+}
 
-	ZERO_STRUCTP(str);
+/*******************************************************************
+********************************************************************/
 
-	offset = 4;
+BOOL svcctl_io_service_description( const char *desc, SERVICE_DESCRIPTION *description, RPC_BUFFER *buffer, int depth )
+{
+        prs_struct *ps = &buffer->prs;
 
-	/* set up string lengths. */
+        prs_debug(ps, depth, desc, "svcctl_io_service_description");
+        depth++;
 
-	str->buf_len = create_rpc_blob(str, blob_length);
-	DEBUG(10, ("init_service_description buffer: Allocated a blob of [%d] \n",str->buf_len));
+	if ( !prs_uint32("unknown", ps, depth, &description->unknown) )
+		return False;
+	if ( !prs_unistr("description", ps, depth, &description->description) )
+		return False;
 
-	if ( str && str->buffer && str->buf_len) {
-		memset(str->buffer,0,str->buf_len);
-		memcpy(str->buffer, &offset, sizeof(uint32));
-		bp = &str->buffer[4];
-		if (service_desc) {
-			rpcstr_push(bp, service_desc,str->buf_len-4,0);
+	return True;
+} 
+
+/*******************************************************************
+********************************************************************/
+
+uint32 svcctl_sizeof_service_description( SERVICE_DESCRIPTION *desc )
+{
+	if ( !desc )
+		return 0;
+
+	/* make sure to include the terminating NULL */
+	return ( sizeof(uint32) + (2*(str_len_uni(&desc->description)+1)) );
+}
+
+/*******************************************************************
+********************************************************************/
+
+static BOOL svcctl_io_action( const char *desc, SC_ACTION *action, prs_struct *ps, int depth )
+{
+
+	prs_debug(ps, depth, desc, "svcctl_io_action");
+	depth++;
+
+	if ( !prs_uint32("type", ps, depth, &action->type) )
+		return False;
+	if ( !prs_uint32("delay", ps, depth, &action->delay) )
+		return False;
+
+	return True;
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_service_fa( const char *desc, SERVICE_FAILURE_ACTIONS *fa, RPC_BUFFER *buffer, int depth )
+{
+        prs_struct *ps = &buffer->prs;
+	int i;
+
+        prs_debug(ps, depth, desc, "svcctl_io_service_description");
+        depth++;
+
+	if ( !prs_uint32("reset_period", ps, depth, &fa->reset_period) )
+		return False;
+
+	if ( !prs_pointer( desc, ps, depth, (void*)&fa->rebootmsg, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2 ) )
+		return False;
+	if ( !prs_pointer( desc, ps, depth, (void*)&fa->command, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2 ) )
+		return False;
+
+	if ( !prs_uint32("num_actions", ps, depth, &fa->num_actions) )
+		return False;
+
+	if ( UNMARSHALLING(ps)) {
+		if (fa->num_actions) {
+			if ( !(fa->actions = TALLOC_ARRAY( get_talloc_ctx(), SC_ACTION, fa->num_actions )) ) {
+				DEBUG(0,("svcctl_io_service_fa: talloc() failure!\n"));
+				return False;
+			}
+		} else {
+			fa->actions = NULL;
 		}
 	}
+
+	for ( i=0; i<fa->num_actions; i++ ) {
+		if ( !svcctl_io_action( "actions", &fa->actions[i], ps, depth ) )
+			return False;
+	}
+
+	return True;
+} 
+
+/*******************************************************************
+********************************************************************/
+
+uint32 svcctl_sizeof_service_fa( SERVICE_FAILURE_ACTIONS *fa)
+{
+	uint32 size = 0;
+
+	if ( !fa )
+		return 0;
+
+	size  = sizeof(uint32) * 2;
+	size += sizeof_unistr2( fa->rebootmsg );
+	size += sizeof_unistr2( fa->command );
+	size += sizeof(SC_ACTION) * fa->num_actions;
+
+	return size;
 }
 
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_r_query_service_config2(const char *desc, SVCCTL_R_QUERY_SERVICE_CONFIG2 *r_u, prs_struct *ps, int depth)
+{
+	if ( !r_u )
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_r_query_service_config2");
+	depth++;
+
+	if ( !prs_align(ps) )
+		return False;
+
+	if (!prs_rpcbuffer("", ps, depth, &r_u->buffer))
+		return False;
+	if(!prs_align(ps))
+		return False;
+
+	if (!prs_uint32("needed", ps, depth, &r_u->needed))
+		return False;
+
+	if(!prs_werror("status", ps, depth, &r_u->status))
+		return False;
+
+	return True;
+}
+
+
 /*******************************************************************
 ********************************************************************/
 
@@ -752,7 +912,7 @@ BOOL svcctl_io_q_query_service_status_ex(const char *desc, SVCCTL_Q_QUERY_SERVIC
 	if(!smb_io_pol_hnd("service_pol", &q_u->handle, ps, depth))
 		return False;
 
-	if(!prs_uint32("info_level", ps, depth, &q_u->info_level))
+	if(!prs_uint32("level", ps, depth, &q_u->level))
 		return False;
 
 	if(!prs_uint32("buffer_size", ps, depth, &q_u->buffer_size))
@@ -771,7 +931,7 @@ BOOL svcctl_io_r_query_service_status_ex(const char *desc, SVCCTL_R_QUERY_SERVIC
 		return False;
 
 	prs_debug(ps, depth, desc, "svcctl_io_r_query_service_status_ex");
-		depth++;
+	depth++;
 
 	if (!prs_rpcbuffer("", ps, depth, &r_u->buffer))
 		return False;
@@ -791,37 +951,183 @@ BOOL svcctl_io_r_query_service_status_ex(const char *desc, SVCCTL_R_QUERY_SERVIC
 /*******************************************************************
 ********************************************************************/
 
-BOOL svcctl_io_r_query_service_config2(const char *desc, SVCCTL_R_QUERY_SERVICE_CONFIG2 *r_u, prs_struct *ps, int depth)
+BOOL svcctl_io_q_lock_service_db(const char *desc, SVCCTL_Q_LOCK_SERVICE_DB *q_u, prs_struct *ps, int depth)
 {
-	if ( !r_u )
+	if (q_u == NULL)
 		return False;
 
-	prs_debug(ps, depth, desc, "svcctl_io_r_query_service_config2");
+	prs_debug(ps, depth, desc, "svcctl_io_q_lock_service_db");
 	depth++;
 
 	if(!prs_align(ps))
 		return False;
 
-	if(!prs_uint32("returned", ps, depth, &r_u->returned))
+	if(!smb_io_pol_hnd("scm_handle", &q_u->handle, ps, depth))
 		return False;
 
-	if (r_u->returned > 4) {
-		if (!prs_uint32("offset", ps, depth, &r_u->offset))
-			return False;
+	return True;
 
-		if ( !prs_pointer( desc, ps, depth, (void**)&r_u->description, sizeof(UNISTR2), (PRS_POINTER_CAST)prs_io_unistr2 ) )
-			return False;
+}
 
-		if(!prs_align(ps))
-			return False;
-	} else {
-		/* offset does double duty here */
-		r_u->offset = 0;
-		if (!prs_uint32("offset", ps, depth, &r_u->offset))
-			return False;
-	}
+/*******************************************************************
+********************************************************************/
 
-	if (!prs_uint32("needed", ps, depth, &r_u->needed))
+BOOL svcctl_io_r_lock_service_db(const char *desc, SVCCTL_R_LOCK_SERVICE_DB *r_u, prs_struct *ps, int depth)
+{
+	if ( !r_u )
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_r_lock_service_db");
+	depth++;
+
+	if(!prs_align(ps))
+		return False;
+
+	if(!smb_io_pol_hnd("lock_handle", &r_u->h_lock, ps, depth))
+		return False;
+
+	if(!prs_werror("status", ps, depth, &r_u->status))
+		return False;
+
+	return True;
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_q_unlock_service_db(const char *desc, SVCCTL_Q_UNLOCK_SERVICE_DB *q_u, prs_struct *ps, int depth)
+{
+	if (q_u == NULL)
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_q_unlock_service_db");
+	depth++;
+
+	if(!prs_align(ps))
+		return False;
+
+	if(!smb_io_pol_hnd("h_lock", &q_u->h_lock, ps, depth))
+		return False;
+
+	return True;
+
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_r_unlock_service_db(const char *desc, SVCCTL_R_UNLOCK_SERVICE_DB *r_u, prs_struct *ps, int depth)
+{
+	if ( !r_u )
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_r_unlock_service_db");
+	depth++;
+
+	if(!prs_align(ps))
+		return False;
+
+	if(!prs_werror("status", ps, depth, &r_u->status))
+		return False;
+
+	return True;
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_q_query_service_sec(const char *desc, SVCCTL_Q_QUERY_SERVICE_SEC *q_u, prs_struct *ps, int depth)
+{
+	if (q_u == NULL)
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_q_query_service_sec");
+	depth++;
+
+	if(!prs_align(ps))
+		return False;
+
+	if(!smb_io_pol_hnd("handle", &q_u->handle, ps, depth))
+		return False;
+	if(!prs_uint32("security_flags", ps, depth, &q_u->security_flags))
+		return False;
+	if(!prs_uint32("buffer_size", ps, depth, &q_u->buffer_size))
+		return False;
+
+	return True;
+
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_r_query_service_sec(const char *desc, SVCCTL_R_QUERY_SERVICE_SEC *r_u, prs_struct *ps, int depth)
+{
+	if ( !r_u )
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_r_query_service_sec");
+	depth++;
+
+	if(!prs_align(ps))
+		return False;
+
+	if (!prs_rpcbuffer("buffer", ps, depth, &r_u->buffer))
+		return False;
+
+	if(!prs_uint32("needed", ps, depth, &r_u->needed))
+		return False;
+
+	if(!prs_werror("status", ps, depth, &r_u->status))
+		return False;
+
+	return True;
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_q_set_service_sec(const char *desc, SVCCTL_Q_SET_SERVICE_SEC *q_u, prs_struct *ps, int depth)
+{
+	if (q_u == NULL)
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_q_set_service_sec");
+	depth++;
+
+	if(!prs_align(ps))
+		return False;
+
+	if(!smb_io_pol_hnd("handle", &q_u->handle, ps, depth))
+		return False;
+	if(!prs_uint32("security_flags", ps, depth, &q_u->security_flags))
+		return False;
+
+	if (!prs_rpcbuffer("buffer", ps, depth, &q_u->buffer))
+		return False;
+
+	if(!prs_align(ps))
+		return False;
+
+	if(!prs_uint32("buffer_size", ps, depth, &q_u->buffer_size))
+		return False;
+
+	return True;
+
+}
+
+/*******************************************************************
+********************************************************************/
+
+BOOL svcctl_io_r_set_service_sec(const char *desc, SVCCTL_R_SET_SERVICE_SEC *r_u, prs_struct *ps, int depth)
+{
+	if ( !r_u )
+		return False;
+
+	prs_debug(ps, depth, desc, "svcctl_io_r_set_service_sec");
+	depth++;
+
+	if(!prs_align(ps))
 		return False;
 
 	if(!prs_werror("status", ps, depth, &r_u->status))
@@ -831,3 +1137,5 @@ BOOL svcctl_io_r_query_service_config2(const char *desc, SVCCTL_R_QUERY_SERVICE_
 }
 
 
+
+
