@@ -525,10 +525,10 @@ static BOOL api_samr_query_usergroups(pipes_struct *p)
 }
 
 /*******************************************************************
- api_samr_query_dom_info
+ api_samr_query_domain_info
  ********************************************************************/
 
-static BOOL api_samr_query_dom_info(pipes_struct *p)
+static BOOL api_samr_query_domain_info(pipes_struct *p)
 {
 	SAMR_Q_QUERY_DOMAIN_INFO q_u;
 	SAMR_R_QUERY_DOMAIN_INFO r_u;
@@ -538,16 +538,16 @@ static BOOL api_samr_query_dom_info(pipes_struct *p)
 	ZERO_STRUCT(q_u);
 	ZERO_STRUCT(r_u);
 
-	if(!samr_io_q_query_dom_info("", &q_u, data, 0)) {
-		DEBUG(0,("api_samr_query_dom_info: unable to unmarshall SAMR_Q_QUERY_DOMAIN_INFO.\n"));
+	if(!samr_io_q_query_domain_info("", &q_u, data, 0)) {
+		DEBUG(0,("api_samr_query_domain_info: unable to unmarshall SAMR_Q_QUERY_DOMAIN_INFO.\n"));
 		return False;
 	}
 
-	r_u.status = _samr_query_dom_info(p, &q_u, &r_u);
+	r_u.status = _samr_query_domain_info(p, &q_u, &r_u);
 
 	/* store the response in the SMB stream */
-	if(!samr_io_r_query_dom_info("", &r_u, rdata, 0)) {
-		DEBUG(0,("api_samr_query_dom_info: unable to marshall SAMR_R_QUERY_DOMAIN_INFO.\n"));
+	if(!samr_io_r_query_domain_info("", &r_u, rdata, 0)) {
+		DEBUG(0,("api_samr_query_domain_info: unable to marshall SAMR_R_QUERY_DOMAIN_INFO.\n"));
 		return False;
 	}
 
@@ -679,6 +679,68 @@ static BOOL api_samr_connect4(pipes_struct *p)
 	return True;
 }
 
+/*******************************************************************
+ api_samr_chgpasswd_user3
+ ********************************************************************/
+
+static BOOL api_samr_chgpasswd_user3(pipes_struct *p)
+{
+	SAMR_Q_CHGPASSWD_USER3 q_u;
+	SAMR_R_CHGPASSWD_USER3 r_u;
+	prs_struct *data = &p->in_data.data;
+	prs_struct *rdata = &p->out_data.rdata;
+
+	ZERO_STRUCT(q_u);
+	ZERO_STRUCT(r_u);
+
+	/* change password request */
+	if (!samr_io_q_chgpasswd_user3("", &q_u, data, 0)) {
+		DEBUG(0,("api_samr_chgpasswd_user3: Failed to unmarshall SAMR_Q_CHGPASSWD_USER3.\n"));
+		return False;
+	}
+
+	r_u.status = _samr_chgpasswd_user3(p, &q_u, &r_u);
+
+	/* store the response in the SMB stream */
+	if(!samr_io_r_chgpasswd_user3("", &r_u, rdata, 0)) {
+		DEBUG(0,("api_samr_chgpasswd_user3: Failed to marshall SAMR_R_CHGPASSWD_USER3.\n" ));
+		return False;
+	}
+
+	return True;
+}
+
+/*******************************************************************
+ api_samr_connect5
+ ********************************************************************/
+
+static BOOL api_samr_connect5(pipes_struct *p)
+{
+	SAMR_Q_CONNECT5 q_u;
+	SAMR_R_CONNECT5 r_u;
+	prs_struct *data = &p->in_data.data;
+	prs_struct *rdata = &p->out_data.rdata;
+
+	ZERO_STRUCT(q_u);
+	ZERO_STRUCT(r_u);
+
+	/* grab the samr open policy */
+	if(!samr_io_q_connect5("", &q_u, data, 0)) {
+		DEBUG(0,("api_samr_connect5: unable to unmarshall SAMR_Q_CONNECT5.\n"));
+		return False;
+	}
+
+	r_u.status = _samr_connect5(p, &q_u, &r_u);
+
+	/* store the response in the SMB stream */
+	if(!samr_io_r_connect5("", &r_u, rdata, 0)) {
+		DEBUG(0,("api_samr_connect5: unable to marshall SAMR_R_CONNECT5.\n"));
+		return False;
+	}
+
+	return True;
+}
+
 /**********************************************************************
  api_samr_lookup_domain
  **********************************************************************/
@@ -787,7 +849,7 @@ static BOOL api_samr_set_userinfo(pipes_struct *p)
 		/* Fix for W2K SP2 */
 		/* what is that status-code ? - gd */
 		if (q_u.switch_value == 0x1a) {
-			setup_fault_pdu(p, NT_STATUS(0x1c000006));
+			setup_fault_pdu(p, NT_STATUS(DCERPC_FAULT_INVALID_TAG));
 			return True;
 		}
 		return False;
@@ -1374,30 +1436,29 @@ static BOOL api_samr_remove_sid_foreign_domain(pipes_struct *p)
 }
 
 /*******************************************************************
- api_samr_query_dom_info
+ api_samr_query_dom_info2
  ********************************************************************/
 
-static BOOL api_samr_unknown_2e(pipes_struct *p)
+static BOOL api_samr_query_domain_info2(pipes_struct *p)
 {
-	SAMR_Q_UNKNOWN_2E q_u;
-	SAMR_R_UNKNOWN_2E r_u;
+	SAMR_Q_QUERY_DOMAIN_INFO2 q_u;
+	SAMR_R_QUERY_DOMAIN_INFO2 r_u;
 	prs_struct *data = &p->in_data.data;
 	prs_struct *rdata = &p->out_data.rdata;
 
 	ZERO_STRUCT(q_u);
 	ZERO_STRUCT(r_u);
 
-	/* grab the samr unknown 8 command */
-	if(!samr_io_q_unknown_2e("", &q_u, data, 0)) {
-		DEBUG(0,("api_samr_unknown_2e: unable to unmarshall SAMR_Q_UNKNOWN_2E.\n"));
+	if(!samr_io_q_query_domain_info2("", &q_u, data, 0)) {
+		DEBUG(0,("api_samr_query_domain_info2: unable to unmarshall SAMR_Q_QUERY_DOMAIN_INFO2.\n"));
 		return False;
 	}
 
-	r_u.status = _samr_unknown_2e(p, &q_u, &r_u);
+	r_u.status = _samr_query_domain_info2(p, &q_u, &r_u);
 
 	/* store the response in the SMB stream */
-	if(!samr_io_r_samr_unknown_2e("", &r_u, rdata, 0)) {
-		DEBUG(0,("api_samr_unknown_2e: unable to marshall SAMR_R_UNKNOWN_2E.\n"));
+	if(!samr_io_r_query_domain_info2("", &r_u, rdata, 0)) {
+		DEBUG(0,("api_samr_query_domain_info2: unable to marshall SAMR_R_QUERY_DOMAIN_INFO2.\n"));
 		return False;
 	}
 
@@ -1434,6 +1495,37 @@ static BOOL api_samr_set_dom_info(pipes_struct *p)
 	return True;
 }
 
+/*******************************************************************
+ api_samr_get_dispenum_index
+ ********************************************************************/
+
+static BOOL api_samr_get_dispenum_index(pipes_struct *p)
+{
+	SAMR_Q_GET_DISPENUM_INDEX q_u;
+	SAMR_R_GET_DISPENUM_INDEX r_u;
+	prs_struct *data = &p->in_data.data;
+	prs_struct *rdata = &p->out_data.rdata;
+
+	ZERO_STRUCT(q_u);
+	ZERO_STRUCT(r_u);
+
+	if(!samr_io_q_get_dispenum_index("", &q_u, data, 0)) {
+		DEBUG(0,("api_samr_get_dispenum_index: unable to unmarshall SAMR_Q_GET_DISPENUM_INDEX.\n"));
+		return False;
+	}
+
+	r_u.status = _samr_get_dispenum_index(p, &q_u, &r_u);
+
+	/* store the response in the SMB stream */
+	if(!samr_io_r_get_dispenum_index("", &r_u, rdata, 0)) {
+		DEBUG(0,("api_samr_get_dispenum_index: unable to marshall SAMR_R_GET_DISPENUM_INDEX.\n"));
+		return False;
+	}
+
+	return True;
+}
+
+
 /*******************************************************************
  array of \PIPE\samr operations
  ********************************************************************/
@@ -1467,7 +1559,7 @@ static struct api_struct api_samr_cmds [] =
       {"SAMR_SET_USERINFO"      , SAMR_SET_USERINFO     , api_samr_set_userinfo     },
       {"SAMR_SET_USERINFO2"     , SAMR_SET_USERINFO2    , api_samr_set_userinfo2    },
       
-      {"SAMR_QUERY_DOMAIN_INFO" , SAMR_QUERY_DOMAIN_INFO, api_samr_query_dom_info   },
+      {"SAMR_QUERY_DOMAIN_INFO" , SAMR_QUERY_DOMAIN_INFO, api_samr_query_domain_info},
       {"SAMR_QUERY_USERGROUPS"  , SAMR_QUERY_USERGROUPS , api_samr_query_usergroups },
       {"SAMR_QUERY_DISPINFO"    , SAMR_QUERY_DISPINFO   , api_samr_query_dispinfo   },
       {"SAMR_QUERY_DISPINFO3"   , SAMR_QUERY_DISPINFO3  , api_samr_query_dispinfo   },
@@ -1490,9 +1582,13 @@ static struct api_struct api_samr_cmds [] =
       {"SAMR_QUERY_SEC_OBJECT"  , SAMR_QUERY_SEC_OBJECT , api_samr_query_sec_obj    },
       {"SAMR_SET_SEC_OBJECT"    , SAMR_SET_SEC_OBJECT   , api_samr_set_sec_obj      },
       {"SAMR_GET_USRDOM_PWINFO" , SAMR_GET_USRDOM_PWINFO, api_samr_get_usrdom_pwinfo},
-      {"SAMR_UNKNOWN_2E"        , SAMR_UNKNOWN_2E       , api_samr_unknown_2e       },
+      {"SAMR_QUERY_DOMAIN_INFO2", SAMR_QUERY_DOMAIN_INFO2, api_samr_query_domain_info2},
       {"SAMR_SET_DOMAIN_INFO"   , SAMR_SET_DOMAIN_INFO  , api_samr_set_dom_info     },
-      {"SAMR_CONNECT4"          , SAMR_CONNECT4         , api_samr_connect4         }
+      {"SAMR_CONNECT4"          , SAMR_CONNECT4         , api_samr_connect4         },
+      {"SAMR_CHGPASSWD_USER3"   , SAMR_CHGPASSWD_USER3  , api_samr_chgpasswd_user3  },
+      {"SAMR_CONNECT5"          , SAMR_CONNECT5         , api_samr_connect5         },
+      {"SAMR_GET_DISPENUM_INDEX", SAMR_GET_DISPENUM_INDEX, api_samr_get_dispenum_index },
+      {"SAMR_GET_DISPENUM_INDEX2",SAMR_GET_DISPENUM_INDEX2, api_samr_get_dispenum_index }
 };
 
 void samr_get_pipe_fns( struct api_struct **fns, int *n_fns )
