@@ -306,7 +306,7 @@ retry:
 
 		tried_closest_dc = True; /* avoid loop */
 
-		if (!ads->config.tried_closest_dc) {
+		if (!ads_closest_dc(ads)) {
 
 			namecache_delete(ads->server.realm, 0x1C);
 			namecache_delete(ads->server.workgroup, 0x1C);
@@ -886,7 +886,7 @@ static NTSTATUS net_ads_join_ok(void)
 		return NT_STATUS_ACCESS_DENIED;
 	}
 
-	net_use_machine_password();
+	net_use_krb_machine_account();
 
 	status = ads_startup(True, &ads);
 	if (!ADS_ERR_OK(status)) {
@@ -1575,6 +1575,9 @@ int net_ads_join(int argc, const char **argv)
 		goto fail;
 	}
 
+	saf_join_store(ads->server.workgroup, ads->config.ldap_server_name);
+	saf_join_store(ads->server.realm, ads->config.ldap_server_name);
+
 	/* Verify that everything is ok */
 
 	if ( net_rpc_join_ok(short_domain_name, ads->config.ldap_server_name, &ads->ldap_ip) != 0 ) {
@@ -1720,7 +1723,7 @@ static int net_ads_dns_register(int argc, const char **argv)
 #endif
 	
 	if (argc > 0) {
-		d_fprintf(stderr, "net ads dns register <name> <ip>\n");
+		d_fprintf(stderr, "net ads dns register\n");
 		return -1;
 	}
 
@@ -2170,7 +2173,7 @@ int net_ads_changetrustpw(int argc, const char **argv)
 		return -1;
 	}
 
-	net_use_machine_password();
+	net_use_krb_machine_account();
 
 	use_in_memory_ccache();
 
