@@ -28,6 +28,8 @@ int net_lookup_usage(int argc, const char **argv)
 "  net lookup kdc [realm]\n\tgives IP of realm's kerberos KDC\n\n"
 "  net lookup dc [domain]\n\tgives IP of domains Domain Controllers\n\n"
 "  net lookup master [domain|wg]\n\tgive IP of master browser\n\n"
+"  net lookup name [name]\n\tLookup name's sid and type\n\n"
+"  net lookup sid [sid]\n\tGive sid's name and type\n\n"
 );
 	return -1;
 }
@@ -60,73 +62,95 @@ static int net_lookup_host(int argc, const char **argv)
 	return 0;
 }
 
-static void print_ldap_srvlist(char *srvlist)
+#ifdef HAVE_ADS
+static void print_ldap_srvlist(struct dns_rr_srv *dclist, int numdcs )
 {
-	char *cur, *next;
 	struct in_addr ip;
-	BOOL printit;
+	int i;
 
-	cur = srvlist;
-	do {
-		next = strchr(cur,':');
-		if (next) *next++='\0';
-		printit = resolve_name(cur, &ip, 0x20);
-		cur=next;
-		next=cur ? strchr(cur,' ') :NULL;
-		if (next)
-			*next++='\0';
-		if (printit)
-			d_printf("%s:%s\n", inet_ntoa(ip), cur?cur:"");
-		cur = next;
-	} while (next);
+	for ( i=0; i<numdcs; i++ ) {
+		if ( resolve_name(dclist[i].hostname, &ip, 0x20) ) {
+			d_printf("%s:%d\n", inet_ntoa(ip), dclist[i].port); 
+		}
+	}
 }
-		
+#endif
 
 static int net_lookup_ldap(int argc, const char **argv)
 {
-#ifdef HAVE_LDAP
-	char *srvlist;
+#ifdef HAVE_ADS
 	const char *domain;
-	int rc;
 	struct in_addr addr;
 	struct hostent *hostent;
+	struct dns_rr_srv *dcs = NULL;
+	int numdcs = 0;
+	char *sitename;
+	TALLOC_CTX *ctx;
+	NTSTATUS status;
 
 	if (argc > 0)
 		domain = argv[0];
 	else
 		domain = opt_target_workgroup;
 
+	sitename = sitename_fetch(domain);
+
+	if ( (ctx = talloc_init("net_lookup_ldap")) == NULL ) {
+		d_fprintf(stderr, "net_lookup_ldap: talloc_inti() failed!\n");
+		SAFE_FREE(sitename);
+		return -1;
+	}
+
 	DEBUG(9, ("Lookup up ldap for domain %s\n", domain));
-	rc = ldap_domain2hostlist(domain, &srvlist);
-	if ((rc == LDAP_SUCCESS) && srvlist) {
-		print_ldap_srvlist(srvlist);
+
+	status = ads_dns_query_dcs( ctx, domain, sitename, &dcs, &numdcs );
+	if ( NT_STATUS_IS_OK(status) && numdcs ) {
+		print_ldap_srvlist(dcs, numdcs);
+		TALLOC_FREE( ctx );
+		SAFE_FREE(sitename);
 		return 0;
 	}
 
      	DEBUG(9, ("Looking up DC for domain %s\n", domain));
-	if (!get_pdc_ip(domain, &addr))
+	if (!get_pdc_ip(domain, &addr)) {
+		TALLOC_FREE( ctx );
+		SAFE_FREE(sitename);
 		return -1;
+	}
 
 	hostent = gethostbyaddr((char *) &addr.s_addr, sizeof(addr.s_addr),
 				AF_INET);
-	if (!hostent)
+	if (!hostent) {
+		TALLOC_FREE( ctx );
+		SAFE_FREE(sitename);
 		return -1;
+	}
 
 	DEBUG(9, ("Found DC with DNS name %s\n", hostent->h_name));
 	domain = strchr(hostent->h_name, '.');
-	if (!domain)
+	if (!domain) {
+		TALLOC_FREE( ctx );
+		SAFE_FREE(sitename);
 		return -1;
+	}
 	domain++;
 
 	DEBUG(9, ("Looking up ldap for domain %s\n", domain));
-	rc = ldap_domain2hostlist(domain, &srvlist);
-	if ((rc == LDAP_SUCCESS) && srvlist) {
-		print_ldap_srvlist(srvlist);
+
+	status = ads_dns_query_dcs( ctx, domain, sitename, &dcs, &numdcs );
+	if ( NT_STATUS_IS_OK(status) && numdcs ) {
+		print_ldap_srvlist(dcs, numdcs);
+		TALLOC_FREE( ctx );
+		SAFE_FREE(sitename);
 		return 0;
 	}
+
+	TALLOC_FREE( ctx );
+	SAFE_FREE(sitename);
+
 	return -1;
 #endif
-	DEBUG(1,("No LDAP support\n"));
+	DEBUG(1,("No ADS support\n"));
 	return -1;
 }
 
@@ -136,6 +160,7 @@ static int net_lookup_dc(int argc, const char **argv)
 	struct in_addr addr;
 	char *pdc_str = NULL;
 	const char *domain=opt_target_workgroup;
+	char *sitename = NULL;
 	int count, i;
 
 	if (argc > 0)
@@ -148,10 +173,13 @@ static int net_lookup_dc(int argc, const char **argv)
 	asprintf(&pdc_str, "%s", inet_ntoa(addr));
 	d_printf("%s\n", pdc_str);
 
-	if (!get_sorted_dc_list(domain, &ip_list, &count, False)) {
+	sitename = sitename_fetch(domain);
+	if (!NT_STATUS_IS_OK(get_sorted_dc_list(domain, sitename, &ip_list, &count, False))) {
 		SAFE_FREE(pdc_str);
+		SAFE_FREE(sitename);
 		return 0;
 	}
+	SAFE_FREE(sitename);
 	for (i=0;i<count;i++) {
 		char *dc_str = inet_ntoa(ip_list[i].ip);
 		if (!strequal(pdc_str, dc_str))
@@ -185,6 +213,7 @@ static int net_lookup_kdc(int argc, const char **argv)
 	krb5_data realm;
 	char **realms;
 
+	initialize_krb5_error_table();
 	rc = krb5_init_context(&ctx);
 	if (rc) {
 		DEBUG(1,("krb5_init_context failed (%s)\n", 
@@ -193,11 +222,11 @@ static int net_lookup_kdc(int argc, const char **argv)
 	}
 
 	if (argc>0) {
-                realm.data = CONST_DISCARD(krb5_pointer, argv[0]);
+                realm.data = CONST_DISCARD(char *, argv[0]);
 		realm.length = strlen(argv[0]);
 	} else if (lp_realm() && *lp_realm()) {
-		realm.data = (krb5_pointer) lp_realm();
-		realm.length = strlen(realm.data);
+		realm.data = lp_realm();
+		realm.length = strlen((const char *)realm.data);
 	} else {
 		rc = krb5_get_host_realm(ctx, NULL, &realms);
 		if (rc) {
@@ -205,13 +234,13 @@ static int net_lookup_kdc(int argc, const char **argv)
 				 error_message(rc)));
 			return -1;
 		}
-		realm.data = (krb5_pointer) *realms;
-		realm.length = strlen(realm.data);
+		realm.data = (char *) *realms;
+		realm.length = strlen((const char *)realm.data);
 	}
 
-	rc = krb5_locate_kdc(ctx, &realm, (struct sockaddr **) &addrs, &num_kdcs, 0);
+	rc = smb_krb5_locate_kdc(ctx, &realm, (struct sockaddr **)(void *)&addrs, &num_kdcs, 0);
 	if (rc) {
-		DEBUG(1, ("krb5_locate_kdc failed (%s)\n", error_message(rc)));
+		DEBUG(1, ("smb_krb5_locate_kdc failed (%s)\n", error_message(rc)));
 		return -1;
 	}
 	for (i=0;i<num_kdcs;i++)
@@ -225,6 +254,54 @@ static int net_lookup_kdc(int argc, const char **argv)
 	return -1;
 }
 
+static int net_lookup_name(int argc, const char **argv)
+{
+	const char *dom, *name;
+	DOM_SID sid;
+	enum lsa_SidType type;
+
+	if (argc != 1) {
+		d_printf("usage: net lookup name <name>\n");
+		return -1;
+	}
+
+	if (!lookup_name(tmp_talloc_ctx(), argv[0], LOOKUP_NAME_ALL,
+			 &dom, &name, &sid, &type)) {
+		d_printf("Could not lookup name %s\n", argv[0]);
+		return -1;
+	}
+
+	d_printf("%s %d (%s) %s\\%s\n", sid_string_static(&sid),
+		 type, sid_type_lookup(type), dom, name);
+	return 0;
+}
+
+static int net_lookup_sid(int argc, const char **argv)
+{
+	const char *dom, *name;
+	DOM_SID sid;
+	enum lsa_SidType type;
+
+	if (argc != 1) {
+		d_printf("usage: net lookup sid <sid>\n");
+		return -1;
+	}
+
+	if (!string_to_sid(&sid, argv[0])) {
+		d_printf("Could not convert %s to SID\n", argv[0]);
+		return -1;
+	}
+
+	if (!lookup_sid(tmp_talloc_ctx(), &sid,
+			&dom, &name, &type)) {
+		d_printf("Could not lookup name %s\n", argv[0]);
+		return -1;
+	}
+
+	d_printf("%s %d (%s) %s\\%s\n", sid_string_static(&sid),
+		 type, sid_type_lookup(type), dom, name);
+	return 0;
+}
 
 /* lookup hosts or IP addresses using internal samba lookup fns */
 int net_lookup(int argc, const char **argv)
@@ -237,6 +314,8 @@ int net_lookup(int argc, const char **argv)
 		{"DC", net_lookup_dc},
 		{"MASTER", net_lookup_master},
 		{"KDC", net_lookup_kdc},
+		{"NAME", net_lookup_name},
+		{"SID", net_lookup_sid},
 		{NULL, NULL}
 	};
 
